;  PROG29D - Working through the Emulator  
;
;  Start with the Serial Interface.
;
;  PROG29D - Start Working with:
;   U - Unassemble (Display Control Store)
;   C - Change Control Store
;   R - Display Scratchpad RAM and SFR
;   W - Write to Scratchpad RAM and SFR
;
;  PROG29C - Start Doing the Data Interface
;
;  PROG29B - Start with the Command Line Interface.  Upon Return, R0 Points to the
;   First Parameter and R1 Points to the Second.  If there is a Third, Error
;
;   Input String Buffer is 10 Long Starting at 080h.
;
;   Commands are:
;    H        - Help, Display this List
;    T        - Reset Microcontroller
;    D        - Download a New Application
;    E [Addr] - Start Running at the Optional Address
;    1 [Addr] - Single Step from the Optional Address
;             - If just a Carriage Return Received then Single Step 
;    U [Addr] - Unassemble at the PC or Optional Address
;    C Addr   - Modify the Control Store at the Address
;    W Addr   - Write to the Scratchpad RAM/SFR Address
;    R        - Display the Scratchpad RAM Registers
;    B Addr   - Toggle Set Breakpoint at the Specified Address
;    L        - List the Breakpoints
;  
;  PROG29A - Develop Serial Interface - Bounce Back Characters from the Host
;   Terminal.  
;
;  The Serial Interface consists of a Command and a Parameter Followed by a
;   Carriage Return, We Have to Interpret this as well as:
;
;   1.  Ignore Following "Line Feed" - Wait 10 msecs after CR.
;   
;   2.  Handle BackSpacing
;
;  Myke Predko
;  98.06.25
;
;  Hardware Notes:
;  DS87C520 is used as the Microcontroller
;   - Oscillator Speed is 20 MHz
;  P2.7 is Console Serial Output
;  P2.6 is Console Serial Input

;  Constant Declarations
define SerOut=P2.7              ;  Serial Output Bit
define SerIn=P2.6               ;  Serial Input Bit
;define SerOut=P1.0              ;  Serial Output Bit
;define SerIn=P1.6               ;  Serial Input Bit

CR     EQU 00Dh                 ;  ASCII Carriage Return
LF     EQU 00Ah                 ;  ASCII Line Feed
BS     EQU 008h                 ;  ASCII BackSpace

CRLF   EQU 005h                 ;  Send CR/LF to the Console

;  Variable Declarations
SavedRegs EQU 080h              ;  Saved Bank Registers and SP, DPTR, IE, PSW, ACC, B  
Buffer EQU 090h                 ;  Input/Breakpoint Buffer Area
                                ;   - 10 Characters Long


;  Macros


;  Mainline
 org 0                          ;  Execution Starts Here
MainLine:                       ;  Mainline of the Program

  mov     A,#0                  ;  Display the Introductory Message
  acall   SendMsg

  acall   DisplayHelp1          ;  As the First Thing, Display Help
  acall   DisplayHelp2
  acall   DisplayHelp3

Loop:

  acall   Prompt                ;  Set the Prompt

  acall   GetCommand            ;  Wait for the Command to Come in

  cjne    R0,#0FFh,Not_Step     ;  Single Step the Processor?

Step:                           ;  If Nothing Entered, Then Step

;  #### - Single Step the Processor (Check R1 for Address)

  ajmp    Loop

Not_Step:                       ;  Look at the Different Commands

  mov     A,@R0                 ;  Get the Command
  xrl     A,#'T'                ;  Reset the Processor?
  jnz     Not_Reset

;  #### - Reset the Processor

  ajmp    Loop

Not_Reset:

  mov     A,@R0                 ;  Download a New Application?
  xrl     A,#'D'
  jnz     Not_DownLoad

;  #### - Download the Application

  ajmp    Loop

Not_Download:                   ;  Execute the Application?

  mov     A,@R0
  xrl     A,#'E'
  jnz     Not_Execute

;  #### - Execute the Application; note that Address May be Specified

  ajmp    Loop

Not_Execute:                    ;  Single Step?

  mov     A,@R0
  xrl     A,#'1'
  jz      Step                  ;  Yes, Jump to Single Step Algorithm

  mov     A,@R0                 ;  Unassemble Some Code?
  xrl     A,#'U'
  jnz     Not_Unassemble

;  #### - Unassemble the code; Note, if No Second Parm, Start at PC

  ajmp    Loop

Not_Unassemble:                 ;  Modify the Control Store at Specified Address

  mov     A,@R0
  xrl     A,#'C'
  jnz     Not_Change

;  #### - Change the Data at the Specified Address

  ajmp    Loop

Not_Change:                     ;  Display Scratchpad RAM/Special Function Registers

  mov     A,@R0
  xrl     A,#'R'
  jnz     Not_RegRead

;  #### - Read and Display the Registers

  ajmp    Loop

Not_RegRead:                    ;  Write to the Scratchpad RAM/Special Function Register

  mov     A,@R0
  xrl     A,#'W'
  jz      NowDo_RegWrite
  ajmp    Not_RegWrite
NowDo_RegWrite:

  cjne    R1,#0FFh,RegWrite     ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  acall   SendMsg

  ajmp    Loop

RegWrite:

  mov     A,R1                  ;  Figure Out if Valid Address
  anl     A,#00Fh               ;  Clear the Buffer Address
  clr     C
  subb    A,R2                  ;  Check the End Address
  add     A,#2
  jc      Do_RegWrite           ;  If Carry Set, then Difference is 1 or 2

  mov     A,#3                  ;  Invalid Address
  acall   SendMsg

  ajmp    Loop

Do_RegWrite:

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,Hex_Skip1	;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
Hex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     R0,A

  mov     A,R1                  ;  Do we Have one or two Characters?
  anl     A,#00Fh
  clr     C
  subb    A,R2
  xrl     A,#0FFh               ;  Invert the Result
  inc     A
  xrl     A,#2
  jnz     HaveAddr

  mov     A,R0
  swap A                        ;  Byte in "A" is Actually High Nybble
  mov     R0,A

  inc     R1                    ;  Get the Second Nybble of the Address
  mov     A,@R1
  clr     C
  jnb     ACC.6,Hex_Skip2
  subb    A,#7
Hex_Skip2:
  subb    A,#030h

  orl     A,R0
  mov     R0,A

HaveAddr:                       ;  Have a Valid Address in R0, Now, Get the New Value

  mov     5,R0                  ;  Save the Address 

  mov     A,#2                  ;  Put in the Prompt Message
  acall   SendMsg

  mov     0,R5                  ;  Restore the Address

  mov     A,#0                  ;  Get the Hex Value
  acall   GetHex
  jnc     Reg_Check             ;  Check the Register to Read
  ajmp    Change_End            ;  If Nothing Entered, then Skip Over Write
Reg_Check:  

  mov     R1,A                  ;  Save the New Value

  mov     A,R0                  ;  Get the Address to Check
  clr     C
  subb    A,#080h               ;  Do We Write to a Scratchpad Register or SFR?
  jnc     SFR_Write             ;  No Carry, A special Function Register

  mov     A,R0                  ;  Now, Do we write to the Bank Registers?
  clr     C
  subb    A,#008h               ;  Do We have a Bank Register?
  jnc     Scratch_Write         ;  No, Can Write Directly

  mov     A,R0
  add     A,#SavedRegs          ;  Put the Bank Registers in the Saved Register Area
  mov     R0,A

Scratch_Write:                  ;  Write to Scratchpad RAM

  mov     A,R1
  mov     @R0,A

  mov     A,#5                  ;  New Line and End
  acall   SendMsg

  ajmp    Loop

SFR_Write:                      ;  Write to the Specified SFR

  mov     A,R0                  ;  Look At SP
  xrl     A,#081h
  jnz     SFRW_NotSP

  mov     R0,#088h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotSP:

  mov     A,R0                  ;  Look At DPL
  xrl     A,#082h
  jnz     SFRW_NotDPL

  mov     R0,#089h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotDPL:

  mov     A,R0                  ;  Look At DPH
  xrl     A,#083h
  jnz     SFRW_NotDPH

  mov     R0,#08Ah              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotDPH:

  mov     A,R0                  ;  Look At PCON
  xrl     A,#087h
  jnz     SFRW_NotPCON

  mov     087h,R1               ;  Write Directly to "PCON"

  ajmp    Change_End

SFRW_NotPCON:

  mov     A,R0                  ;  Look At TCON
  xrl     A,#088h
  jnz     SFRW_NotTCON

  mov     088h,R1               ;  Write Directly to "TCON"

  ajmp    Change_End

SFRW_NotTCON:

  mov     A,R0                  ;  Look At TMOD
  xrl     A,#089h
  jnz     SFRW_NotTMOD

  mov     089h,R1               ;  Write Directly to "TMOD"

  ajmp    Change_End

SFRW_NotTMOD:

  mov     A,R0                  ;  Look At TL0
  xrl     A,#08Ah
  jnz     SFRW_NotTL0

  mov     08Ah,R1               ;  Write Directly to "TL0"

  ajmp    Change_End

SFRW_NotTL0:

  mov     A,R0                  ;  Look At TL1
  xrl     A,#08Bh
  jnz     SFRW_NotTL1

  mov     08Bh,R1               ;  Write Directly to "TL1"

  ajmp    Change_End

SFRW_NotTL1:

  mov     A,R0                  ;  Look At TH0
  xrl     A,#08Ch
  jnz     SFRW_NotTH0

  mov     08Ch,R1               ;  Write Directly to "TH0"

  ajmp    Change_End

SFRW_NotTH0:

  mov     A,R0                  ;  Look At TH1
  xrl     A,#08Dh
  jnz     SFRW_NotTH1

  mov     08Dh,R1               ;  Write Directly to "TH1"

  ajmp    Change_End

SFRW_NotTH1:

  mov     A,R0                  ;  Look At P1
  xrl     A,#090h
  jnz     SFRW_NotP1

  mov     090h,R1               ;  Write Directly to "P1"

  ajmp    Change_End

SFRW_NotP1:

  mov     A,R0                  ;  Look At SCON
  xrl     A,#098h
  jnz     SFRW_NotSCON

  mov     098h,R1               ;  Write Directly to "SCON"

  ajmp    Change_End

SFRW_NotSCON:

  mov     A,R0                  ;  Look At SBUF
  xrl     A,#099h
  jnz     SFRW_NotSBUF

  mov     099h,R1               ;  Write Directly to "SBUF"

  ajmp    Change_End

SFRW_NotSBUF:

  mov     A,R0                  ;  Look At IE
  xrl     A,#0A8h
  jnz     SFRW_NotIE

  mov     R0,#08Bh              ;  Save IE in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotIE:

  mov     A,R0                  ;  Look At P3
  xrl     A,#0B0h
  jnz     SFRW_NotP3

  mov     0B0h,R1               ;  Write Directly to "P3"

  ajmp    Change_End

SFRW_NotP3:

  mov     A,R0                  ;  Look At IP
  xrl     A,#0B8h
  jnz     SFRW_NotIP

  mov     0B8h,R1               ;  Write Directly to "IP"

  ajmp    Change_End

SFRW_NotIP:

  mov     A,R0                  ;  Look At PSW
  xrl     A,#0D0h
  jnz     SFRW_NotPSW

  mov     R0,#08Ch              ;  Save PSW in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotPSW:

  mov     A,R0                  ;  Look At ACC
  xrl     A,#0E0h
  jnz     SFRW_NotACC

  mov     R0,#08Dh              ;  Save ACC in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotACC:

  mov     A,R0                  ;  Look At "B"
  xrl     A,#0F0h
  jnz     SFRW_NotB

  mov     R0,#08Eh              ;  Save "B" in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotB:

  mov     A,#3                  ;  Not a Valid AT89Cx051 Message
  acall   SendMsg

  ajmp    Loop

Change_End:

  mov     A,#CRLF               ;  Start a New Line
  acall   SendMsg

  ajmp    Loop

Not_RegWrite:                   ;  Toggle Breakpoint at the Specified Address

  mov     A,@R0
  xrl     A,#'B'
  jnz     Not_BreakPoint

;  #### - Toggle the Breakpoint at the Specified Address

  ajmp    Loop

Not_BreakPoint:                 ;  List the Breakpoints set in the Emulator?

  mov     A,@R0
  xrl     A,#'L'
  jnz     Check_Help

;  #### - Display the Breakpoints at the Specified Address

  ajmp    Loop

Check_Help:                     ;  Display the Help Message

  mov     A,@R0                 ;  Check for "H"elp Selected
  xrl     A,#'H'
  jz      Help

  mov     A,#1                  ;  Return "Invalid Message"
  acall   SendMsg

Help:

  acall   DisplayHelp1          ;  As the First Thing, Display Help
  acall   DisplayHelp2
  acall   DisplayHelp3

  ajmp    Loop  


;  Subroutines
Prompt:                         ;  Display a Prompt for the Program

;  #### - Put in Processor Information Here

  mov     A,#'>'
  acall   SendChar

  mov     A,#' '
  acall   SendChar

  ret


GetHex:                         ;  Get a Hex Byte from the User

  mov     R2,#0                 ;  Character to Return
  mov     R3,A                  ;  Save the Return Type 
                                ;   !0 - Space or Enter
                                ;    0 - Just Enter
  mov     R4,#0                 ;  Number of Characters Read

GetHex_Loop:                    ;  Loop Here for Each Character

  acall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetHex_Loop           ;  If Carry Set, Nothing to Check

  mov     R5,A                  ;  Save the Character Returned

  xrl     A,#00Dh               ;  Was Enter Pressed?  
  jnz     GetHex_NotEnter

  acall   GetChar               ;  Wait for LF (if Coming

GetHex_Enter:

  mov     A,R2                  ;  Get the Number to Return

  clr     C                     ;  Assume Something Returned, Reset Carry
  cjne    R4,#0,GetHex_End      ;  Do we Have a number to Return?
  setb    C                     ;  Nothing Entered, Set Carry 
GetHex_End:

  ret

GetHex_NotEnter:                ;  Not Enter, What Else is it?

  mov     A,R5                  ;  Do We have a Blank?  
  xrl     A,#' '
  jnz     GetHex_NotBlank

  cjne    R3,#0,GetHex_Enter    ;  Can we Return on a Blank?
  
  ajmp    GetHex_Loop           ;  Else, Ignore a Loop

GetHex_NotBlank:                ;  Now, Look For Valid Hex

  mov     A,R5                  ;  Do we Have a Backspace?
  xrl     A,#BS
  jnz     GetHex_NotBS

  mov     A,R4                  ;  Do we Have a Character to Take Away?
  jz      GetHex_Loop

  mov     A,#BS                 ;  Return the BS to the Host
  acall   SendChar

  mov     A,R2                  ;  Take One Digit out of the Return Value
  swap A
  anl     A,#00Fh               ;  Clear the Upper 4 Bits of Shifted Result
  mov     R2,A

  dec     R4

  ajmp    GetHex_Loop

GetHex_NotBS:                   ;  Have Ascii Character?

  mov     A,R4                  ;  Do we Have 2 Characters Already?  
  xrl     A,#2
  jz      GetHex_Loop           ;  Yes, Ignore the Character

  mov     A,R5                  ;  Do Hex Comparison
  clr     C
  subb    A,#'0'
  jc      GetHex_Loop           ;  Less than '0', Not Valid Hex

  mov     A,R5
  clr     C
  subb    A,#('9' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "A" to "F"?
  clr     C
  subb    A,#'A'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('F' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "a" to "f"?
  clr     C
  subb    A,#'a'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('f' + 1)
  jnc     GetHex_Loop

  mov     A,R5
  anl     A,#05Fh               ;  Convert Number to Upper Case
  mov     R5,A

GetHex_HaveHex:                 ;  Have a Hex Character in R5, Add to R2

  mov     A,R5                  ;  Send the Character Back to the Host
  acall   SendChar

  mov     A,R2                  ;  Prepare R2 for the Character
  swap A
  mov     R2,A

  mov     A,R5
  clr	  C			
  jnb	  ACC.6,GetHex_Skip  		;  If Bit 6 of the ASCII Value is Set
  subb    A,#7				;   Subtract Difference between 'A' & '9'
GetHex_Skip:
  subb    A,#030h			;  Take away '0' to get the Nybble

  orl     A,R2                          ;  Add the Bits to R2
  mov     R2,A

  inc     R4                    ;  Increment the Number of Characters  

  ajmp    GetHex_Loop


GetCommand:                     ;  Get the Command from the User

  mov     R2,#0                 ;  R3 Counts the Number of Characters Entered
  mov     R3,#' '               ;  R4 Has the Last Character Entered
                                ;   Use "Blank" incase a Character Entered
  mov     R4,#0FFh              ;  R5 Has the Position of the First Parameter
  mov     R5,#0FFh              ;  R6 Has the Position of the Second Parameter

GetCommand_Loop:                ;  Loop Around Here

  acall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetCommand_Loop       ;  If Carry Set, Nothing to Check

  cjne    A,#CR,GetCommand_Test ;  Look for a Carriage Return

  acall   GetChar               ;  Wait for/Skip the LF

  mov     A,#CRLF               ;  Start a New Line
  acall   SendMsg

  mov     0,R4                  ;  Save the Parameter Position Values
  mov     1,R5

  mov     A,R4                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R4                  ;  Pass Back the Parameters
  add     A,#Buffer
  mov     R0,A

  mov     A,R5                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R5
  add     A,#Buffer
  mov     R1,A

GetCommand_End:

  ret

GetCommand_Test:                ;  Now, Process the Character

  mov     R1,A                  ;  Save the Character

  clr     C                     ;  Do we Have a Special Character?
  subb    A,#' '                ;  Is there anything less than ' '?

  jnc     GetCommand_Char       ;  No, Process the Character

  cjne    R1,#BS,GetCommand_Loop;  If it's not BackSpace, Display the Character
  
  cjne    R2,#0,GetCommand_BS   ;  If No Characters, Then Skip
  ajmp    GetCommand_Loop
GetCommand_BS:

  mov     A,R1                  ;  Send the BackSpace Character
  acall   SendChar

  dec     R2                    ;  Decrement the Displayed Characters

  mov     A,R2                  ;  Now, Do We Have to Clear the Parameter Pos'ns?
  xrl     A,R5                  ;  Check the Second Parameter
  jnz     GetCommand_BSFirst

  mov     R5,#0FFh              ;  Reset the Second Parameter

GetCommand_BSFirst:             ;  Check the position against the First Parameter

  mov     A,R2
  xrl     A,R4
  jnz     GetCommand_BSNew      

  mov     R4,#0FFh

GetCommand_BSNew:               ;  Now, Finish off with Resetting the Last Character

  mov     R3,#' '               ;  Get the Last Character
  
  mov     A,R2
  jz      GetCommand_Loop       ;  If At the Start of the Row, Return

  add     A,#Buffer             ;  Get the Actual Position from the Buffer
  dec     A                     ;   Of the Previous Character

  mov     R0,A                  ;  Get the Character and Save it in R3
  mov     A,@R0
  mov     R3,A

  ajmp    GetCommand_Loop

GetCommand_Char:                ;  Process the Character

  cjne    R2,#10,GetCommand_OK  ;  If 10 Characters In, 
  ajmp    GetCommand_Loop
GetCommand_OK:

  mov     A,R1                  ;  Make Sure the Character is Upper Case
  clr     C
  subb    A,#'a'                ;  Are we less than 'a'?
  jc      GetCommand_UC

  mov     A,R1                  ;  Are we Greater than 'z'?
  clr     C
  subb    A,#('z' + 1)
  jnc     GetCommand_UC

  mov     A,R1
  anl     A,#05Fh               ;  Lower Case, Convert to Upper
  mov     R1,A

GetCommand_UC:                  ;  Have an Upper Case Character

  cjne    R3,#' ',GetCommand_Do ;  If Previous == ' ', Save the Parameter?
  mov     A,R1
  xrl     A,#' '
  jz      GetCommand_Do         ;  If Current is Also ' ', Don't Save Parameter

  cjne    R4,#0FFh,GetCommand_2nd

  mov     4,R2                  ;  Save the Position for the First Parameter

  ajmp    GetCommand_Do

GetCommand_2nd:                 ;  Save the Parameter as 2nd?

  cjne    R5,#0FFh,GetCommand_Do

  mov     5,R2                  ;   Yes

GetCommand_Do:                  ;  Save the Character and Wait for Next

  cjne    R5,#0FFh,GetCommand_2ndHex

GetCommand_ActualDo:

  mov     A,R2                  ;  Save the Character
  add     A,#Buffer
  mov     R0,A
  mov     A,R1
  mov     @R0,A

  mov     A,R1
  mov     R3,A                  ;  Save the Character as Previous
  acall   SendChar              ;  Send the Character to the Display

  inc     R2                    ;  Point to the Next Position

  ajmp    GetCommand_Loop

GetCommand_2ndHex:              ;  Second Parameter, Must Be Hex (and Max of 4 Chars)

  mov     A,R2                  ;  Do we Have 4 Characters?  
  clr     C
  subb    A,R5
  xrl     A,#3
  jnz     GetCommand_ActualDo   ;  No, Add to the Buffer

  ajmp    GetCommand_Loop       ;  Yes, Don't Add Any More 


DisplayHelp1:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable1 - GetHelp1)

DisplayHelp_Loop1:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp1:

  jz      DisplayHelp_End1      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop1     ;  Loop Around to Display the Next Character
  
DisplayHelp_End1:

  ret

HelpTable1:                     ;  Messages to be Displayed
  db      CR,LF                 ;  Start a New Line
  db      'H        - Help, Display this List',CR,LF
  db      'T        - Reset',CR,LF
  db      'D        - Download a New Application',CR,LF
  db      'E [Addr] - Execute at the Optional Address',CR,LF
  db      '1 [Addr] - Single Step from the Optional Address',CR,LF,0
DisplayHelp2:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop2:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp2:

  jz      DisplayHelp_End2      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop2     ;  Loop Around to Display the Next Character
  
DisplayHelp_End2:

  ret

HelpTable2:                     ;  Messages to be Displayed
  db      '         - If just a Carriage Return Received then Single Step',CR,LF
  db      'U [Addr] - Unassemble at the PC or Optional Address',CR,LF
  db      'C Addr   - Modify the Control Store at the Address',CR,LF,0
DisplayHelp3:

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop3:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp3:

  jz      DisplayHelp_End3      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop3     ;  Loop Around to Display the Next Character
  
DisplayHelp_End3:

  ret

HelpTable3:                     ;  Messages to be Displayed
  db      'W Addr   - Write to the Scratchpad RAM/SFR Address',CR,LF
  db      'R        - Display the Scratchpad RAM Registers',CR,LF
  db      'B Addr   - Toggle Set Breakpoint at the Specified Address',CR,LF
  db      'L        - List the Breakpoints',CR,LF,0


GetChar:                        ;  Wait for a Character and Return it in Accumulator

  mov  R7,#28                   ;  Delay 10 msecs for Test
  mov  R6,#0
GCLoop1:			;  Wait 10 msecs for a Start Bit
  jnb  SerIn,GCSkip
  djnz R6,GCLoop1
  djnz R7,GCLoop1

  setb C                        ;  Nothing Received, Set Carry to Indicate

  ret

GCSkip:                         ;  Have Something to Receive

  mov  R6,#9	                ;  Want to Read 9 Bits

  mov  R7,#70		        ;  Delay Less than 1/2 Bit to Make Up for Calls, Etc.
GCLoop2:
  djnz R7,GCLoop2

  jb   SerIn,GCLoop1	        ;  If "Glitch", keep Waiting

GCLoop:			        ;  Now, Read 9 Bits

  mov  C,SerIn	                ;  Read the Input Bit
  rrc  A

  mov  R7,#170  	        ;  Delay Loop for 104 usecs per bit
GCLoop3:
  djnz R7,GCLoop3
  nop
  nop

  djnz R6,GCLoop

  clr  C                        ;  Make Sure Carry is Clear

  ret


SendChar:                       ;  Send the Serial Character
  mov	 R6,#10		        ;  We're Actually Sending 10 Bits

  clr	 C	       	        ;  First Bit is a "0"

SCLoop:			        ;  Loop Here for Each Bit

  mov	 SerOut,C               ;  Output the Bit in the Carry Flag

  mov	 R7,#170	        ;  Delay to 104 usecs (520 Cycles) per loop
SCLoop1:
  djnz	 R7,SCLoop1
  nop

  setb	 C		        ;  Shift the Next Bit to Carry
  rrc 	 A		        ;   And Load in a "1" for Stop Bit

  djnz	 R6,SCLoop

  ret


SendMsg:                        ;  Send the Specified Message to the Display

  mov     R1,A                  ;  Save the Message Number

  mov     R0,#0                 ;  Use R0 as the Offset Counter

SendMsg_Find:                   ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      SendMsg_Output        ;   If Count is Zero

  mov     A,#LOW(SendMsgTable - SendMsgFindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
SendMsgFindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     SendMsg_Find          ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  ajmp    SendMsg_Find

SendMsg_Output:                 ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(SendMsgTable - SendMsgOutputGet)

  movc A,@A+PC                  ;  Read the Table
SendMsgOutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      SendMsg_End           ;  If at Zero, Stop the Reading/Displaying

  acall   SendChar              ;  Output the Character

  ajmp    SendMsg_Output

SendMsg_End:                    ;  Message Output 

  ret

SendMsgTable:                   ;  Messages to be Displayed
  db      CR,LF,'AT89Cx051 Emulator',CR,LF,0        ;  Message  0
  db      'Invalid Command',CR,LF,0                 ;  Message  1
  db      'Enter New Value ',0                      ;  Message  2
  db      'Invalid Address',CR,LF,0                 ;  Message  3
  db      'No Address Specified',CR,LF,0            ;  Message  4
  db      CR,LF,0                                   ;  Message  5
