;  PROG29E - Working through the Emulator  
;
;  Start with the Serial Interface.
;
;  PROG29E - Display Scratchpad/SFR Data Registers
;
;  PROG29D - Start Working with:
;   U - Unassemble (Display Control Store)
;   C - Change Control Store
;   R - Display Scratchpad RAM and SFR
;   W - Write to Scratchpad RAM and SFR
;
;  PROG29C - Start Doing the Data Interface
;
;  PROG29B - Start with the Command Line Interface.  Upon Return, R0 Points to the
;   First Parameter and R1 Points to the Second.  If there is a Third, Error
;
;   Input String Buffer is 10 Long Starting at 080h.
;
;   Commands are:
;    H        - Help, Display this List
;    T        - Reset Microcontroller
;    D        - Download a New Application
;    E [Addr] - Start Running at the Optional Address
;    1 [Addr] - Single Step from the Optional Address
;             - If just a Carriage Return Received then Single Step 
;    U [Addr] - Unassemble at the PC or Optional Address
;    C Addr   - Modify the Control Store at the Address
;    W Addr   - Write to the Scratchpad RAM/SFR Address
;    R        - Display the Scratchpad RAM Registers
;    B Addr   - Toggle Set Breakpoint at the Specified Address
;    L        - List the Breakpoints
;  
;  PROG29A - Develop Serial Interface - Bounce Back Characters from the Host
;   Terminal.  
;
;  The Serial Interface consists of a Command and a Parameter Followed by a
;   Carriage Return, We Have to Interpret this as well as:
;
;   1.  Ignore Following "Line Feed" - Wait 10 msecs after CR.
;   
;   2.  Handle BackSpacing
;
;  Myke Predko
;  98.06.27
;
;  Hardware Notes:
;  DS87C520 is used as the Microcontroller
;   - Oscillator Speed is 20 MHz
;  P2.7 is Console Serial Output
;  P2.6 is Console Serial Input

;  Constant Declarations
define SerOut=P2.7              ;  Serial Output Bit
define SerIn=P2.6               ;  Serial Input Bit
;define SerOut=P1.0              ;  Serial Output Bit
;define SerIn=P1.6               ;  Serial Input Bit

CR        EQU 00Dh              ;  ASCII Carriage Return
LF        EQU 00Ah              ;  ASCII Line Feed
BS        EQU 008h              ;  ASCII BackSpace

CRLF      EQU 005h              ;  Send CR/LF to the Console

;  Variable Declarations
SavedRegs EQU 080h              ;  Saved Bank Registers and SP, DPTR, IE, PSW, ACC, B  
Buffer    EQU 090h              ;  Input/Breakpoint Buffer Area
                                ;   - 10 Characters Long
ProgC     EQU 0A0h              ;  Program Counter for Emulated Device


;  Macros


;  Mainline
 org 0                          ;  Execution Starts Here
MainLine:                       ;  Mainline of the Program

  mov     A,#0                  ;  Display the Introductory Message
  acall   SendMsg

  acall   DisplayHelp1          ;  As the First Thing, Display Help
  acall   DisplayHelp2
  acall   DisplayHelp3

Loop:

  acall   Prompt                ;  Set the Prompt

  acall   GetCommand            ;  Wait for the Command to Come in

  cjne    R0,#0FFh,Not_Step     ;  Single Step the Processor?

Step:                           ;  If Nothing Entered, Then Step

;  #### - Single Step the Processor (Check R1 for Address)

  ajmp    Loop

Not_Step:                       ;  Look at the Different Commands

  mov     A,@R0                 ;  Get the Command
  xrl     A,#'T'                ;  Reset the Processor?
  jnz     Not_Reset

;  #### - Reset the Processor

  ajmp    Loop

Not_Reset:

  mov     A,@R0                 ;  Download a New Application?
  xrl     A,#'D'
  jnz     Not_DownLoad

;  #### - Download the Application

  ajmp    Loop

Not_Download:                   ;  Execute the Application?

  mov     A,@R0
  xrl     A,#'E'
  jnz     Not_Execute

;  #### - Execute the Application; note that Address May be Specified

  ajmp    Loop

Not_Execute:                    ;  Single Step?

  mov     A,@R0
  xrl     A,#'1'
  jz      Step                  ;  Yes, Jump to Single Step Algorithm

  mov     A,@R0                 ;  Unassemble Some Code?
  xrl     A,#'U'
  jz      Unassemble
  ajmp    Not_Unassemble
Unassemble:

  cjne    R1,#0FFh,GetUAddr     ;  Is there a Valid Address to Write to?

  mov     R0,#ProgC             ;  Read the Program Counter into DPTR
  mov     A,@R0
  mov     DPL,A
  inc     R0
  mov     A,@R0
  mov     DPH,A

  ajmp    HaveUAddr

GetUAddr:                       ;  Get the Address Specified from the Command

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

UAddr_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,UHex_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
UHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     UAddr_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      HaveUAddr

  mov     A,#3                  ;  Put in the Invalid Address Error
  acall   SendMsg

  ajmp    Loop

HaveUAddr:

  mov     A,DPH                 ;  Put the Address in the External RAM
  add     A,#040h               ;   Starts at 16K
  mov     DPH,A

  mov     R5,#23                ;  Display 23 Lines

HUA_Loop:                       ;  Loop For Each Instruction

  movx A,@DPTR                  ;  Read the Value from External Memory

  mov     R4,A                  ;  Save the Instruction for Later

  lcall   InsLength             ;  Get the Instruction Length
  mov     R3,A

HUA_Loop1:                      ;  Now, Display Each Instruction 

  movx A,@DPTR                  ;  Get the Byte to Display
  acall   SendHex

  mov     A,#' '
  acall   SendChar

  inc     DPTR

  djnz    R3,HUA_Loop1

;  #### - Put in the Actual Instruction

  mov     A,#CRLF
  acall   SendMsg

  djnz    R5,HUA_Loop

  ajmp    Loop

Not_Unassemble:                 ;  Modify the Control Store at Specified Address

  mov     A,@R0
  xrl     A,#'C'
  jz      Change
  ajmp    Not_Change
Change:

  cjne    R1,#0FFh,C_RegWrite   ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  acall   SendMsg

  ajmp    Loop

C_RegWrite:

  mov     DPL,#0                ;  Store the Address in DPTR
  mov     DPH,#0

  mov     A,R2                  ;  Make R2 Based on the Buffer
  add     A,#Buffer
  mov     R2,A

CAddr_Loop:                     ;  Loop Here Until R1 == R2

  mov     A,DPH                 ;  Shift DPTR up by 4 for Each Loop
  swap A
  anl     A,#0F0h               ;  Shift up DPH
  mov     DPH,A

  mov     A,DPL                 ;  Add the High 4 Bits of DPL to DPH
  swap A
  anl     A,#00Fh
  orl     A,DPH
  mov     DPH,A

  mov     A,DPL                 ;  Shift Up the Low 4 Bits of DPL
  swap A
  anl     A,#0F0h
  mov     DPL,A

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,CHex_Skip1      ;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
CHex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  orl     A,DPL
  mov     DPL,A                 ;  Save the Nybble in DPL

  inc     R1                    ;  Point to the Next Character

  mov     A,R2                  ;  Now, Are we At the End?
  xrl     A,R1
  jnz     CAddr_Loop            ;  If R1 != R2, Then Get the Next Character

  mov     A,DPH                 ;  Do we Have a Valid Address (to 0800h)?
  clr     C
  subb    A,#008h
  jc      CAddr_OK

  mov     A,#3                  ;  Put in the Invalid Address Error
  acall   SendMsg

  ajmp    Loop

CAddr_OK:                       ;  The Address is OK - Now Get the New Value

  mov     A,DPH                 ;  Put the Address in the External RAM
  add     A,#040h               ;   Starts at 16K
  mov     DPH,A

  mov     A,#2                  ;  Put in the Prompt Message
  acall   SendMsg

  mov     A,#0                  ;  Get the Hex Value
  acall   GetHex
  jnc     CReg_Check            ;  Check the Register to Read
  ajmp    Change_End            ;  If Nothing Entered, then Skip Over Write
CReg_Check:  

  movx @DPTR,A                  ;  Save the New Value in External Memory

  mov     A,#CRLF
  acall   SendMsg

  ajmp    Loop

Not_Change:                     ;  Display Scratchpad RAM/Special Function Registers

  mov     A,@R0
  xrl     A,#'R'
  jz      RegRead
  ajmp    Not_RegRead
RegRead:

  mov     A,#6                  ;  Display the Scratchpad Registers First
  acall   SendMsg

  mov     A,#0                  ;  Display the Saved Scratchpad RAM Registers First
  acall   SendHex

  mov     A,#7                  ;  ":  "
  acall   SendMsg

  mov     R0,#SavedRegs         ;  use "R0" as the Index Into Saved Regs
  mov     R1,#8                 ;  Do 8 Registers First
  
RR_Loop1:                       ;  Do the Line

  mov     A,@R0                 ;  Get the First Byte
  acall   SendHex

  mov     A,#' '
  acall   SendChar

  inc     R0                    ;  Point to the Next Register

  djnz    R1,RR_Loop1

  mov     R0,#008h              ;  Now, Display the Rest of the Scratchpad RAM

  mov     R1,#008h              ;  End with the Rest of the First Line
 
RR_Loop2:

  mov     A,@R0
  acall   SendHex

  mov     A,#' '
  acall   SendChar

  inc     R0

  djnz    R1,RR_Loop2

  mov     2,R0                  ;  Save Address for Later

  mov     A,#CRLF               ;  Now, Go to another line
  acall   SendMsg

  mov     A,R2                  ;  Get the Address

  jb      ACC.7,ScratchDone     ;  If Bit 7 is Set, then Scratchpad is Done

  mov     A,R2                  ;  Otherwise, Start a New Line
  acall   SendHex

  mov     A,#7
  acall   SendMsg

  mov     0,R2                  ;  Do the Next Line
  mov     R1,#16                ;  Each Line 16 Bytes Long

  ajmp    RR_Loop2

ScratchDone:                    ;  Scratchpad Registers Displayed

  mov     A,#CRLF               ;  Space Out by One Line
  acall   SendMsg

  mov     A,#22                 ;  First, Display the "B" Register
  acall   SendMsg

  mov     R0,#08Eh
  mov     A,@R0
  acall   SendHex               ;  Send the "B" Register Contents

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#21                 ;  Next, Display the PSW Contents
  acall   SendMsg

  mov     R0,#08Ch
  mov     A,@R0
  acall   SendBit

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#8                  ;  Display the SP Contents
  acall   SendMsg

  mov     R0,#088h
  mov     A,@R0
  acall   SendHex

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#9                  ;  Display the DPTR Contents
  acall   SendMsg

  mov     R0,#08Ah
  mov     A,@R0
  acall   SendHex
  dec     R0
  mov     A,@R0
  acall   SendHex

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#19                  ;  Display the Interrupt Controller Contents
  acall   SendMsg

  mov     R0,#08Bh
  mov     A,@R0
  acall   SendBit

  mov     A,#' '
  acall   SendChar
  mov     A,#' '
  acall   SendChar

  mov     A,#20
  acall   SendMsg

  mov     A,IP
  acall   SendBit

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#10                  ;  Display the PCON Contents
  acall   SendMsg

  mov     A,PCON
  acall   SendBit

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#15                  ;  Display I/O Port Values
  acall   SendMsg

  mov     A,P1
  acall   SendBit

  mov     A,#' '
  acall   SendChar
  mov     A,#' '
  acall   SendChar

  mov     A,#16
  acall   SendMsg

  mov     A,P3
  acall   SendBit

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#11                  ;  Display Timer Control Values
  acall   SendMsg

  mov     A,TCON
  acall   SendBit

  mov     A,#' '
  acall   SendChar
  mov     A,#' '
  acall   SendChar

  mov     A,#12
  acall   SendMsg

  mov     A,TMOD
  acall   SendBit

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#13                  ;  Display the Timer 0 Contents
  acall   SendMsg

  mov     A,TH0
  acall   SendHex
  mov     A,TL0
  acall   SendHex

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#14                  ;  Display the Timer 1 Contents
  acall   SendMsg

  mov     A,TH1
  acall   SendHex
  mov     A,TL1
  acall   SendHex

  mov     A,#CRLF
  acall   SendMsg

  mov     A,#17                  ;  Display Serial Port Values
  acall   SendMsg

  mov     A,SCON
  acall   SendBit

  mov     A,#' '
  acall   SendChar
  mov     A,#' '
  acall   SendChar

  mov     A,#18
  acall   SendMsg

  mov     A,SBUF
  acall   SendHex

  mov     A,#CRLF
  acall   SendMsg

  ajmp    Loop

Not_RegRead:                    ;  Write to the Scratchpad RAM/Special Function Register

  mov     A,@R0
  xrl     A,#'W'
  jz      NowDo_RegWrite
  ajmp    Not_RegWrite
NowDo_RegWrite:

  cjne    R1,#0FFh,RegWrite     ;  Is there a Valid Address to Write to?

  mov     A,#4                  ;  No Address Specified
  acall   SendMsg

  ajmp    Loop

RegWrite:

  mov     A,R1                  ;  Figure Out if Valid Address
  anl     A,#00Fh               ;  Clear the Buffer Address
  clr     C
  subb    A,R2                  ;  Check the End Address
  add     A,#2
  jc      Do_RegWrite           ;  If Carry Set, then Difference is 1 or 2

  mov     A,#3                  ;  Invalid Address
  acall   SendMsg

  ajmp    Loop

Do_RegWrite:

  mov     A,@R1                 ;  Convert the Address
  clr	  C                     ;   To a Hex Value			
  jnb	  ACC.6,Hex_Skip1	;  If Bit 6 of the ASCII Value is Set
  subb    A,#7		 	;   Subtract Difference between 'A' & '9'
Hex_Skip1:
  subb    A,#030h	 	;  Take away '0' to get the Nybble
  mov     R0,A

  mov     A,R1                  ;  Do we Have one or two Characters?
  anl     A,#00Fh
  clr     C
  subb    A,R2
  xrl     A,#0FFh               ;  Invert the Result
  inc     A
  xrl     A,#2
  jnz     HaveAddr

  mov     A,R0
  swap A                        ;  Byte in "A" is Actually High Nybble
  mov     R0,A

  inc     R1                    ;  Get the Second Nybble of the Address
  mov     A,@R1
  clr     C
  jnb     ACC.6,Hex_Skip2
  subb    A,#7
Hex_Skip2:
  subb    A,#030h

  orl     A,R0
  mov     R0,A

HaveAddr:                       ;  Have a Valid Address in R0, Now, Get the New Value

  mov     5,R0                  ;  Save the Address 

  mov     A,#2                  ;  Put in the Prompt Message
  acall   SendMsg

  mov     0,R5                  ;  Restore the Address

  mov     A,#0                  ;  Get the Hex Value
  acall   GetHex
  jnc     Reg_Check             ;  Check the Register to Read
  ajmp    Change_End            ;  If Nothing Entered, then Skip Over Write
Reg_Check:  

  mov     R1,A                  ;  Save the New Value

  mov     A,R0                  ;  Get the Address to Check
  clr     C
  subb    A,#080h               ;  Do We Write to a Scratchpad Register or SFR?
  jnc     SFR_Write             ;  No Carry, A special Function Register

  mov     A,R0                  ;  Now, Do we write to the Bank Registers?
  clr     C
  subb    A,#008h               ;  Do We have a Bank Register?
  jnc     Scratch_Write         ;  No, Can Write Directly

  mov     A,R0
  add     A,#SavedRegs          ;  Put the Bank Registers in the Saved Register Area
  mov     R0,A

Scratch_Write:                  ;  Write to Scratchpad RAM

  mov     A,R1
  mov     @R0,A

  mov     A,#5                  ;  New Line and End
  acall   SendMsg

  ajmp    Loop

SFR_Write:                      ;  Write to the Specified SFR

  mov     A,R0                  ;  Look At SP
  xrl     A,#081h
  jnz     SFRW_NotSP

  mov     R0,#088h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotSP:

  mov     A,R0                  ;  Look At DPL
  xrl     A,#082h
  jnz     SFRW_NotDPL

  mov     R0,#089h              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotDPL:

  mov     A,R0                  ;  Look At DPH
  xrl     A,#083h
  jnz     SFRW_NotDPH

  mov     R0,#08Ah              ;  Save SP in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotDPH:

  mov     A,R0                  ;  Look At PCON
  xrl     A,#087h
  jnz     SFRW_NotPCON

  mov     087h,R1               ;  Write Directly to "PCON"

  ajmp    Change_End

SFRW_NotPCON:

  mov     A,R0                  ;  Look At TCON
  xrl     A,#088h
  jnz     SFRW_NotTCON

  mov     088h,R1               ;  Write Directly to "TCON"

  ajmp    Change_End

SFRW_NotTCON:

  mov     A,R0                  ;  Look At TMOD
  xrl     A,#089h
  jnz     SFRW_NotTMOD

  mov     089h,R1               ;  Write Directly to "TMOD"

  ajmp    Change_End

SFRW_NotTMOD:

  mov     A,R0                  ;  Look At TL0
  xrl     A,#08Ah
  jnz     SFRW_NotTL0

  mov     08Ah,R1               ;  Write Directly to "TL0"

  ajmp    Change_End

SFRW_NotTL0:

  mov     A,R0                  ;  Look At TL1
  xrl     A,#08Bh
  jnz     SFRW_NotTL1

  mov     08Bh,R1               ;  Write Directly to "TL1"

  ajmp    Change_End

SFRW_NotTL1:

  mov     A,R0                  ;  Look At TH0
  xrl     A,#08Ch
  jnz     SFRW_NotTH0

  mov     08Ch,R1               ;  Write Directly to "TH0"

  ajmp    Change_End

SFRW_NotTH0:

  mov     A,R0                  ;  Look At TH1
  xrl     A,#08Dh
  jnz     SFRW_NotTH1

  mov     08Dh,R1               ;  Write Directly to "TH1"

  ajmp    Change_End

SFRW_NotTH1:

  mov     A,R0                  ;  Look At P1
  xrl     A,#090h
  jnz     SFRW_NotP1

  mov     090h,R1               ;  Write Directly to "P1"

  ajmp    Change_End

SFRW_NotP1:

  mov     A,R0                  ;  Look At SCON
  xrl     A,#098h
  jnz     SFRW_NotSCON

  mov     098h,R1               ;  Write Directly to "SCON"

  ajmp    Change_End

SFRW_NotSCON:

  mov     A,R0                  ;  Look At SBUF
  xrl     A,#099h
  jnz     SFRW_NotSBUF

  mov     099h,R1               ;  Write Directly to "SBUF"

  ajmp    Change_End

SFRW_NotSBUF:

  mov     A,R0                  ;  Look At IE
  xrl     A,#0A8h
  jnz     SFRW_NotIE

  mov     R0,#08Bh              ;  Save IE in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotIE:

  mov     A,R0                  ;  Look At P3
  xrl     A,#0B0h
  jnz     SFRW_NotP3

  mov     0B0h,R1               ;  Write Directly to "P3"

  ajmp    Change_End

SFRW_NotP3:

  mov     A,R0                  ;  Look At IP
  xrl     A,#0B8h
  jnz     SFRW_NotIP

  mov     0B8h,R1               ;  Write Directly to "IP"

  ajmp    Change_End

SFRW_NotIP:

  mov     A,R0                  ;  Look At PSW
  xrl     A,#0D0h
  jnz     SFRW_NotPSW

  mov     R0,#08Ch              ;  Save PSW in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotPSW:

  mov     A,R0                  ;  Look At ACC
  xrl     A,#0E0h
  jnz     SFRW_NotACC

  mov     R0,#08Dh              ;  Save ACC in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotACC:

  mov     A,R0                  ;  Look At "B"
  xrl     A,#0F0h
  jnz     SFRW_NotB

  mov     R0,#08Eh              ;  Save "B" in "SavedRegs"
  mov     A,R1
  mov     @R0,A

  ajmp    Change_End

SFRW_NotB:

  mov     A,#3                  ;  Not a Valid AT89Cx051 Message
  acall   SendMsg

  ajmp    Loop

Change_End:

  mov     A,#CRLF               ;  Start a New Line
  acall   SendMsg

  ajmp    Loop

Not_RegWrite:                   ;  Toggle Breakpoint at the Specified Address

  mov     A,@R0
  xrl     A,#'B'
  jnz     Not_BreakPoint

;  #### - Toggle the Breakpoint at the Specified Address

  ajmp    Loop

Not_BreakPoint:                 ;  List the Breakpoints set in the Emulator?

  mov     A,@R0
  xrl     A,#'L'
  jnz     Check_Help

;  #### - Display the Breakpoints at the Specified Address

  ajmp    Loop

Check_Help:                     ;  Display the Help Message

  mov     A,@R0                 ;  Check for "H"elp Selected
  xrl     A,#'H'
  jz      Help

  mov     A,#1                  ;  Return "Invalid Message"
  acall   SendMsg

Help:

  acall   DisplayHelp1          ;  As the First Thing, Display Help
  acall   DisplayHelp2
  acall   DisplayHelp3

  ajmp    Loop  


;  Subroutines
Prompt:                         ;  Display a Prompt for the Program

;  #### - Put in Processor Information Here

  mov     A,#'>'
  acall   SendChar

  mov     A,#' '
  acall   SendChar

  ret


GetHex:                         ;  Get a Hex Byte from the User

  mov     R2,#0                 ;  Character to Return
  mov     R3,A                  ;  Save the Return Type 
                                ;   !0 - Space or Enter
                                ;    0 - Just Enter
  mov     R4,#0                 ;  Number of Characters Read

GetHex_Loop:                    ;  Loop Here for Each Character

  acall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetHex_Loop           ;  If Carry Set, Nothing to Check

  mov     R5,A                  ;  Save the Character Returned

  xrl     A,#00Dh               ;  Was Enter Pressed?  
  jnz     GetHex_NotEnter

  acall   GetChar               ;  Wait for LF (if Coming

GetHex_Enter:

  mov     A,R2                  ;  Get the Number to Return

  clr     C                     ;  Assume Something Returned, Reset Carry
  cjne    R4,#0,GetHex_End      ;  Do we Have a number to Return?
  setb    C                     ;  Nothing Entered, Set Carry 
GetHex_End:

  ret

GetHex_NotEnter:                ;  Not Enter, What Else is it?

  mov     A,R5                  ;  Do We have a Blank?  
  xrl     A,#' '
  jnz     GetHex_NotBlank

  cjne    R3,#0,GetHex_Enter    ;  Can we Return on a Blank?
  
  ajmp    GetHex_Loop           ;  Else, Ignore a Loop

GetHex_NotBlank:                ;  Now, Look For Valid Hex

  mov     A,R5                  ;  Do we Have a Backspace?
  xrl     A,#BS
  jnz     GetHex_NotBS

  mov     A,R4                  ;  Do we Have a Character to Take Away?
  jz      GetHex_Loop

  mov     A,#BS                 ;  Return the BS to the Host
  acall   SendChar

  mov     A,R2                  ;  Take One Digit out of the Return Value
  swap A
  anl     A,#00Fh               ;  Clear the Upper 4 Bits of Shifted Result
  mov     R2,A

  dec     R4

  ajmp    GetHex_Loop

GetHex_NotBS:                   ;  Have Ascii Character?

  mov     A,R4                  ;  Do we Have 2 Characters Already?  
  xrl     A,#2
  jz      GetHex_Loop           ;  Yes, Ignore the Character

  mov     A,R5                  ;  Do Hex Comparison
  clr     C
  subb    A,#'0'
  jc      GetHex_Loop           ;  Less than '0', Not Valid Hex

  mov     A,R5
  clr     C
  subb    A,#('9' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "A" to "F"?
  clr     C
  subb    A,#'A'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('F' + 1)
  jc      GetHex_HaveHex

  mov     A,R5                  ;  "a" to "f"?
  clr     C
  subb    A,#'a'
  jc      GetHex_Loop

  mov     A,R5
  clr     C
  subb    A,#('f' + 1)
  jnc     GetHex_Loop

  mov     A,R5
  anl     A,#05Fh               ;  Convert Number to Upper Case
  mov     R5,A

GetHex_HaveHex:                 ;  Have a Hex Character in R5, Add to R2

  mov     A,R5                  ;  Send the Character Back to the Host
  acall   SendChar

  mov     A,R2                  ;  Prepare R2 for the Character
  swap A
  mov     R2,A

  mov     A,R5
  clr	  C			
  jnb	  ACC.6,GetHex_Skip  		;  If Bit 6 of the ASCII Value is Set
  subb    A,#7				;   Subtract Difference between 'A' & '9'
GetHex_Skip:
  subb    A,#030h			;  Take away '0' to get the Nybble

  orl     A,R2                          ;  Add the Bits to R2
  mov     R2,A

  inc     R4                    ;  Increment the Number of Characters  

  ajmp    GetHex_Loop


GetCommand:                     ;  Get the Command from the User

  mov     R2,#0                 ;  R3 Counts the Number of Characters Entered
  mov     R3,#' '               ;  R4 Has the Last Character Entered
                                ;   Use "Blank" incase a Character Entered
  mov     R4,#0FFh              ;  R5 Has the Position of the First Parameter
  mov     R5,#0FFh              ;  R6 Has the Position of the Second Parameter

GetCommand_Loop:                ;  Loop Around Here

  acall   GetChar               ;  Get Character/Wait 10 msecs

  jc      GetCommand_Loop       ;  If Carry Set, Nothing to Check

  cjne    A,#CR,GetCommand_Test ;  Look for a Carriage Return

  acall   GetChar               ;  Wait for/Skip the LF

  mov     A,#CRLF               ;  Start a New Line
  acall   SendMsg

  mov     0,R4                  ;  Save the Parameter Position Values
  mov     1,R5

  mov     A,R4                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R4                  ;  Pass Back the Parameters
  add     A,#Buffer
  mov     R0,A

  mov     A,R5                  ;  Do We Have Something to Change?
  xrl     A,#0FFh
  jz      GetCommand_End

  mov     A,R5
  add     A,#Buffer
  mov     R1,A

GetCommand_End:

  ret

GetCommand_Test:                ;  Now, Process the Character

  mov     R1,A                  ;  Save the Character

  clr     C                     ;  Do we Have a Special Character?
  subb    A,#' '                ;  Is there anything less than ' '?

  jnc     GetCommand_Char       ;  No, Process the Character

  cjne    R1,#BS,GetCommand_Loop;  If it's not BackSpace, Display the Character
  
  cjne    R2,#0,GetCommand_BS   ;  If No Characters, Then Skip
  ajmp    GetCommand_Loop
GetCommand_BS:

  mov     A,R1                  ;  Send the BackSpace Character
  acall   SendChar

  dec     R2                    ;  Decrement the Displayed Characters

  mov     A,R2                  ;  Now, Do We Have to Clear the Parameter Pos'ns?
  xrl     A,R5                  ;  Check the Second Parameter
  jnz     GetCommand_BSFirst

  mov     R5,#0FFh              ;  Reset the Second Parameter

GetCommand_BSFirst:             ;  Check the position against the First Parameter

  mov     A,R2
  xrl     A,R4
  jnz     GetCommand_BSNew      

  mov     R4,#0FFh

GetCommand_BSNew:               ;  Now, Finish off with Resetting the Last Character

  mov     R3,#' '               ;  Get the Last Character
  
  mov     A,R2
  jz      GetCommand_Loop       ;  If At the Start of the Row, Return

  add     A,#Buffer             ;  Get the Actual Position from the Buffer
  dec     A                     ;   Of the Previous Character

  mov     R0,A                  ;  Get the Character and Save it in R3
  mov     A,@R0
  mov     R3,A

  ajmp    GetCommand_Loop

GetCommand_Char:                ;  Process the Character

  cjne    R2,#10,GetCommand_OK  ;  If 10 Characters In, 
  ajmp    GetCommand_Loop
GetCommand_OK:

  mov     A,R1                  ;  Make Sure the Character is Upper Case
  clr     C
  subb    A,#'a'                ;  Are we less than 'a'?
  jc      GetCommand_UC

  mov     A,R1                  ;  Are we Greater than 'z'?
  clr     C
  subb    A,#('z' + 1)
  jnc     GetCommand_UC

  mov     A,R1
  anl     A,#05Fh               ;  Lower Case, Convert to Upper
  mov     R1,A

GetCommand_UC:                  ;  Have an Upper Case Character

  cjne    R3,#' ',GetCommand_Do ;  If Previous == ' ', Save the Parameter?
  mov     A,R1
  xrl     A,#' '
  jz      GetCommand_Do         ;  If Current is Also ' ', Don't Save Parameter

  cjne    R4,#0FFh,GetCommand_2nd

  mov     4,R2                  ;  Save the Position for the First Parameter

  ajmp    GetCommand_Do

GetCommand_2nd:                 ;  Save the Parameter as 2nd?

  cjne    R5,#0FFh,GetCommand_Do

  mov     5,R2                  ;   Yes

GetCommand_Do:                  ;  Save the Character and Wait for Next

  cjne    R5,#0FFh,GetCommand_2ndHex

GetCommand_ActualDo:

  mov     A,R2                  ;  Save the Character
  add     A,#Buffer
  mov     R0,A
  mov     A,R1
  mov     @R0,A

  mov     A,R1
  mov     R3,A                  ;  Save the Character as Previous
  acall   SendChar              ;  Send the Character to the Display

  inc     R2                    ;  Point to the Next Position

  ajmp    GetCommand_Loop

GetCommand_2ndHex:              ;  Second Parameter, Must Be Hex (and Max of 4 Chars)

  mov     A,R2                  ;  Do we Have 4 Characters?  
  clr     C
  subb    A,R5
  xrl     A,#3
  jnz     GetCommand_ActualDo   ;  No, Add to the Buffer

  ajmp    GetCommand_Loop       ;  Yes, Don't Add Any More 


DisplayHelp1:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable1 - GetHelp1)

DisplayHelp_Loop1:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp1:

  jz      DisplayHelp_End1      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop1     ;  Loop Around to Display the Next Character
  
DisplayHelp_End1:

  ret

HelpTable1:                     ;  Messages to be Displayed
  db      CR,LF                 ;  Start a New Line
  db      'H        - Help, Display this List',CR,LF
  db      'T        - Reset',CR,LF
  db      'D        - Download a New Application',CR,LF
  db      'E [Addr] - Execute at the Optional Address',CR,LF
  db      '1 [Addr] - Single Step from the Optional Address',CR,LF,0
DisplayHelp2:                   ;  Display the "Help" Menu

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop2:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp2:

  jz      DisplayHelp_End2      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop2     ;  Loop Around to Display the Next Character
  
DisplayHelp_End2:

  ret

HelpTable2:                     ;  Messages to be Displayed
  db      '         - If just a Carriage Return Received then Single Step',CR,LF
  db      'U [Addr] - Unassemble at the PC or Optional Address',CR,LF
  db      'C Addr   - Modify the Control Store at the Address',CR,LF,0
DisplayHelp3:

  mov     R0,#(HelpTable2 - GetHelp2)

DisplayHelp_Loop3:              ;  Loop Here to Display Every Character
  mov     A,R0                  ;   Get the Next Character to Display
  movc A,@A+PC
GetHelp3:

  jz      DisplayHelp_End3      ;  '\0' is the End Character

  acall   SendChar              ;  Send the Character to the Console

  inc     R0                    ;  Point to the Next Character to Display

  ajmp    DisplayHelp_Loop3     ;  Loop Around to Display the Next Character
  
DisplayHelp_End3:

  ret

HelpTable3:                     ;  Messages to be Displayed
  db      'W Addr   - Write to the Scratchpad RAM/SFR Address',CR,LF
  db      'R        - Display the Scratchpad RAM Registers',CR,LF
  db      'B Addr   - Toggle Set Breakpoint at the Specified Address',CR,LF
  db      'L        - List the Breakpoints',CR,LF,0


GetChar:                        ;  Wait for a Character and Return it in Accumulator

  mov  R7,#28                   ;  Delay 10 msecs for Test
  mov  R6,#0
GCLoop1:			;  Wait 10 msecs for a Start Bit
  jnb  SerIn,GCSkip
  djnz R6,GCLoop1
  djnz R7,GCLoop1

  setb C                        ;  Nothing Received, Set Carry to Indicate

  ret

GCSkip:                         ;  Have Something to Receive

  mov  R6,#9	                ;  Want to Read 9 Bits

  mov  R7,#70		        ;  Delay Less than 1/2 Bit to Make Up for Calls, Etc.
GCLoop2:
  djnz R7,GCLoop2

  jb   SerIn,GCLoop1	        ;  If "Glitch", keep Waiting

GCLoop:			        ;  Now, Read 9 Bits

  mov  C,SerIn	                ;  Read the Input Bit
  rrc  A

  mov  R7,#170  	        ;  Delay Loop for 104 usecs per bit
GCLoop3:
  djnz R7,GCLoop3
  nop
  nop

  djnz R6,GCLoop

  clr  C                        ;  Make Sure Carry is Clear

  ret


SendBit:                        ;  Send the Character in "A" as Binary

  mov    R2,A                   ;  Save the Character

  mov    R3,#8                  ;  Do 8 Bits

SB_Loop:                        ;  Loop Here

  mov    A,R2
  rlc A                         ;  Shift the Character Up and Put the LSB Bit in Carry
  mov    R2,A

  mov    A,#'0'                 ;  Figure out what Character
  addc   A,#0                   ;  Add the Carry to ASCII Zero

  acall  SendChar               ;  Send the Bit to the Console

  djnz   R3,SB_Loop

  ret


SendHex:                        ;  Send the Character in "A" as Hex

  mov    R2,A                   ;  Save the Character

  swap A                        ;  Display the High Nybble First
  anl    A,#00Fh
  add    A,#036h	 	;  If ( A & 0x00Fh ) + 6 > 0x010h
  jnb    AC,SH_Skip1	 
  add    A,#7		 	;   Add 7 to Jump to "A" to "F"
SH_Skip1:
  subb   A,#6			;  Take Away Flag Set

  acall  SendChar 

  mov    A,R2                   ;  Do the Lower Nybble   
  anl    A,#00Fh
  add    A,#036h	 	;  If ( A & 0x00Fh ) + 6 > 0x010h
  jnb    AC,SH_Skip2	 
  add    A,#7		 	;   Add 7 to Jump to "A" to "F"
SH_Skip2:
  subb   A,#6			;  Take Away Flag Set

  acall  SendChar

  ret


SendChar:                       ;  Send the Serial Character
  mov	 R6,#10		        ;  We're Actually Sending 10 Bits

  clr	 C	       	        ;  First Bit is a "0"

SCLoop:			        ;  Loop Here for Each Bit

  mov	 SerOut,C               ;  Output the Bit in the Carry Flag

  mov	 R7,#170	        ;  Delay to 104 usecs (520 Cycles) per loop
SCLoop1:
  djnz	 R7,SCLoop1
  nop

  setb	 C		        ;  Shift the Next Bit to Carry
  rrc 	 A		        ;   And Load in a "1" for Stop Bit

  djnz	 R6,SCLoop

  ret


SendMsg:                        ;  Send the Specified Message to the Display

  mov     R1,A                  ;  Save the Message Number

  mov     R0,#0                 ;  Use R0 as the Offset Counter

SendMsg_Find:                   ;  Look for the Appropriate Message

  mov     A,R1                  ;  Are we Now At Zero?
  jz      SendMsg_Output        ;   If Count is Zero

  mov     A,#LOW(SendMsgTable - SendMsgFindGet)
  add     A,R0                  ;  Get Current Address

  movc A,@A+PC                  ;  Get the Character in the Displays
SendMsgFindGet:

  inc     R0                    ;  Point to the Next Character

  jnz     SendMsg_Find          ;  If NOT Equal to Zero,
  
  dec     R1                    ;  It is, Decrement the Message Counter

  ajmp    SendMsg_Find

SendMsg_Output:                 ;  Now, Output the String to '\0'

  mov     A,R0
  add     A,#LOW(SendMsgTable - SendMsgOutputGet)

  movc A,@A+PC                  ;  Read the Table
SendMsgOutputGet:

  inc     R0                    ;  Point to the Next Character

  jz      SendMsg_End           ;  If at Zero, Stop the Reading/Displaying

  acall   SendChar              ;  Output the Character

  ajmp    SendMsg_Output

SendMsg_End:                    ;  Message Output 

  ret

SendMsgTable:                   ;  Messages to be Displayed
  db      CR,LF,'AT89C2051 Emulator',CR,LF,0        ;  Message  0
  db      'Invalid Command',CR,LF,0                 ;  Message  1
  db      'Enter New Value ',0                      ;  Message  2
  db      'Invalid Address',CR,LF,0                 ;  Message  3
  db      'No Address Specified',CR,LF,0            ;  Message  4
  db      CR,LF,0                                   ;  Message  5
  db      'Scratchpad RAM:',CR,LF,0                 ;  Message  6
  db      ':  ',0                                   ;  Message  7
  db      'SP = ',0                                 ;  Message  8  Order  3
  db      'DPTR = ',0                               ;  Message  9  Order  4
  db      'PCON = ',0                               ;  Message 10  Order  6
  db      'TCON = ',0                               ;  Message 11  Order  8
  db      'TMOD = ',0                               ;  Message 12  Order  8
  db      'TH0/TL0 = ',0                            ;  Message 13  Order  9
  db      'TH1/TL1 = ',0                            ;  Message 14  Order 10
  db      'P1 = ',0                                 ;  Message 15  Order  7
  db      'P3 = ',0                                 ;  Message 16  Order  7
  db      'SCON = ',0                               ;  Message 17  Order 11
  db      'SBUF = ',0                               ;  Message 18  Order 11
  db      'IE = ',0                                 ;  Message 19  Order  5
  db      'IP = ',0                                 ;  Message 20  Order  5
  db      'PSW = ',0                                ;  Message 21  Order  2
  db      'B = ',0                                  ;  Message 22  Order  1



InsLength:                      ;  Get the Length for Each Byte

  inc     A                     ;  Increment to get Table Offset
  jnz     GetInsLength          ;   Do we Have 0FFh?

  mov     A,#1                  ;  Yes, Return "1"

  ret

GetInsLength:

  movc A,@A+PC                  ;  Read the Table

  ret

InsLengthTable:                 ;  Length of Each First Byte Instruction
  db      1                     ;  000h - Nop   - One Byte Long
  db      2                     ;  001h - ajmp  - One Byte Long
  db      3                     ;  002h - ljmp  - One Byte Long
  db      1                     ;  003h - rr    - One Byte Long
  db      1                     ;  004h - inc   - One Byte Long
  db      2                     ;  005h - inc   - One Byte Long
  db      1                     ;  006h - inc   - One Byte Long
  db      1                     ;  007h - inc   - One Byte Long
  db      1                     ;  008h - inc   - One Byte Long
  db      1                     ;  009h - inc   - One Byte Long
  db      1                     ;  00Ah - inc   - One Byte Long
  db      1                     ;  00Bh - inc   - One Byte Long
  db      1                     ;  00Ch - inc   - One Byte Long
  db      1                     ;  00Dh - inc   - One Byte Long
  db      1                     ;  00Eh - inc   - One Byte Long
  db      1                     ;  00Fh - inc   - One Byte Long
  db      3                     ;  010h - jbc   - One Byte Long
  db      2                     ;  011h - acall - One Byte Long
  db      3                     ;  012h - lcall - One Byte Long
  db      1                     ;  013h - rrc   - One Byte Long
  db      1                     ;  014h - dec   - One Byte Long
  db      2                     ;  015h - dec   - One Byte Long
  db      1                     ;  016h - dec   - One Byte Long
  db      1                     ;  017h - dec   - One Byte Long
  db      1                     ;  018h - dec   - One Byte Long
  db      1                     ;  019h - dec   - One Byte Long
  db      1                     ;  01Ah - dec   - One Byte Long
  db      1                     ;  01Bh - dec   - One Byte Long
  db      1                     ;  01Ch - dec   - One Byte Long
  db      1                     ;  01Dh - dec   - One Byte Long
  db      1                     ;  01Eh - dec   - One Byte Long
  db      1                     ;  01Fh - dec   - One Byte Long
  db      3                     ;  020h - jb    - One Byte Long
  db      2                     ;  021h - ajmp  - One Byte Long
  db      1                     ;  022h - ret   - One Byte Long
  db      1                     ;  023h - rl    - One Byte Long
  db      2                     ;  024h - add   - One Byte Long
  db      2                     ;  025h - add   - One Byte Long
  db      1                     ;  026h - add   - One Byte Long
  db      1                     ;  027h - add   - One Byte Long
  db      1                     ;  028h - add   - One Byte Long
  db      1                     ;  029h - add   - One Byte Long
  db      1                     ;  02Ah - add   - One Byte Long
  db      1                     ;  02Bh - add   - One Byte Long
  db      1                     ;  02Ch - add   - One Byte Long
  db      1                     ;  02Dh - add   - One Byte Long
  db      1                     ;  02Eh - add   - One Byte Long
  db      1                     ;  02Fh - add   - One Byte Long
  db      3                     ;  030h - jnb   - One Byte Long
  db      2                     ;  031h - acall - One Byte Long
  db      1                     ;  032h - reti  - One Byte Long
  db      1                     ;  033h - rlc   - One Byte Long
  db      2                     ;  034h - addc  - One Byte Long
  db      2                     ;  035h - addc  - One Byte Long
  db      1                     ;  036h - addc  - One Byte Long
  db      1                     ;  037h - addc  - One Byte Long
  db      1                     ;  038h - addc  - One Byte Long
  db      1                     ;  039h - addc  - One Byte Long
  db      1                     ;  03Ah - addc  - One Byte Long
  db      1                     ;  03Bh - addc  - One Byte Long
  db      1                     ;  03Ch - addc  - One Byte Long
  db      1                     ;  03Dh - addc  - One Byte Long
  db      1                     ;  03Eh - addc  - One Byte Long
  db      1                     ;  03Fh - addc  - One Byte Long
  db      2                     ;  040h - jc    - One Byte Long
  db      2                     ;  041h - ajmp  - One Byte Long
  db      2                     ;  042h - orl   - One Byte Long
  db      3                     ;  043h - orl   - One Byte Long
  db      2                     ;  044h - orl   - One Byte Long
  db      2                     ;  045h - orl   - One Byte Long
  db      1                     ;  046h - orl   - One Byte Long
  db      1                     ;  047h - orl   - One Byte Long
  db      1                     ;  048h - orl   - One Byte Long
  db      1                     ;  049h - orl   - One Byte Long
  db      1                     ;  04Ah - orl   - One Byte Long
  db      1                     ;  04Bh - orl   - One Byte Long
  db      1                     ;  04Ch - orl   - One Byte Long
  db      1                     ;  04Dh - orl   - One Byte Long
  db      1                     ;  04Eh - orl   - One Byte Long
  db      1                     ;  04Fh - orl   - One Byte Long
  db      2                     ;  050h - jnc   - One Byte Long
  db      2                     ;  051h - acall - One Byte Long
  db      2                     ;  052h - anl   - One Byte Long
  db      3                     ;  053h - anl   - One Byte Long
  db      2                     ;  054h - anl   - One Byte Long
  db      2                     ;  055h - anl   - One Byte Long
  db      1                     ;  056h - anl   - One Byte Long
  db      1                     ;  057h - anl   - One Byte Long
  db      1                     ;  058h - anl   - One Byte Long
  db      1                     ;  059h - anl   - One Byte Long
  db      1                     ;  05Ah - anl   - One Byte Long
  db      1                     ;  05Bh - anl   - One Byte Long
  db      1                     ;  05Ch - anl   - One Byte Long
  db      1                     ;  05Dh - anl   - One Byte Long
  db      1                     ;  05Eh - anl   - One Byte Long
  db      1                     ;  05Fh - anl   - One Byte Long
  db      2                     ;  060h - jz    - One Byte Long
  db      2                     ;  061h - ajmp  - One Byte Long
  db      2                     ;  062h - xrl   - One Byte Long
  db      3                     ;  063h - xrl   - One Byte Long
  db      2                     ;  064h - xrl   - One Byte Long
  db      2                     ;  065h - xrl   - One Byte Long
  db      1                     ;  066h - xrl   - One Byte Long
  db      1                     ;  067h - xrl   - One Byte Long
  db      1                     ;  068h - xrl   - One Byte Long
  db      1                     ;  069h - xrl   - One Byte Long
  db      1                     ;  06Ah - xrl   - One Byte Long
  db      1                     ;  06Bh - xrl   - One Byte Long
  db      1                     ;  06Ch - xrl   - One Byte Long
  db      1                     ;  06Dh - xrl   - One Byte Long
  db      1                     ;  06Eh - xrl   - One Byte Long
  db      1                     ;  06Fh - xrl   - One Byte Long
  db      2                     ;  070h - jnz   - One Byte Long
  db      2                     ;  071h - acall - One Byte Long
  db      2                     ;  072h - orl   - One Byte Long
  db      1                     ;  073h - jmp   - One Byte Long
  db      2                     ;  074h - mov   - One Byte Long
  db      3                     ;  075h - mov   - One Byte Long
  db      2                     ;  076h - mov   - One Byte Long
  db      2                     ;  077h - mov   - One Byte Long
  db      2                     ;  078h - mov   - One Byte Long
  db      2                     ;  079h - mov   - One Byte Long
  db      2                     ;  07Ah - mov   - One Byte Long
  db      2                     ;  07Bh - mov   - One Byte Long
  db      2                     ;  07Ch - mov   - One Byte Long
  db      2                     ;  07Dh - mov   - One Byte Long
  db      2                     ;  07Eh - mov   - One Byte Long
  db      2                     ;  07Fh - mov   - One Byte Long
  db      2                     ;  080h - sjmp  - One Byte Long
  db      2                     ;  081h - ajmp  - One Byte Long
  db      2                     ;  082h - anl   - One Byte Long
  db      1                     ;  083h - movc  - One Byte Long
  db      1                     ;  084h - div   - One Byte Long
  db      3                     ;  085h - mov   - One Byte Long
  db      2                     ;  086h - mov   - One Byte Long
  db      2                     ;  087h - mov   - One Byte Long
  db      2                     ;  088h - mov   - One Byte Long
  db      2                     ;  089h - mov   - One Byte Long
  db      2                     ;  08Ah - mov   - One Byte Long
  db      2                     ;  08Bh - mov   - One Byte Long
  db      2                     ;  08Ch - mov   - One Byte Long
  db      2                     ;  08Dh - mov   - One Byte Long
  db      2                     ;  08Eh - mov   - One Byte Long
  db      2                     ;  08Fh - mov   - One Byte Long
  db      3                     ;  090h - mov   - One Byte Long
  db      2                     ;  091h - acall - One Byte Long
  db      2                     ;  092h - mov   - One Byte Long
  db      1                     ;  093h - movc  - One Byte Long
  db      2                     ;  094h - subb  - One Byte Long
  db      2                     ;  095h - subb  - One Byte Long
  db      1                     ;  096h - subb  - One Byte Long
  db      1                     ;  097h - subb  - One Byte Long
  db      1                     ;  098h - subb  - One Byte Long
  db      1                     ;  099h - subb  - One Byte Long
  db      1                     ;  09Ah - subb  - One Byte Long
  db      1                     ;  09Bh - subb  - One Byte Long
  db      1                     ;  09Ch - subb  - One Byte Long
  db      1                     ;  09Dh - subb  - One Byte Long
  db      1                     ;  09Eh - subb  - One Byte Long
  db      1                     ;  09Fh - subb  - One Byte Long
  db      2                     ;  0A0h - orl   - One Byte Long
  db      2                     ;  0A1h - ajmp  - One Byte Long
  db      2                     ;  0A2h - mov   - One Byte Long
  db      1                     ;  0A3h - inc   - One Byte Long
  db      1                     ;  0A4h - mul   - One Byte Long
  db      1                     ;  0A5h
  db      2                     ;  0A6h - mov   - One Byte Long
  db      2                     ;  0A7h - mov   - One Byte Long
  db      2                     ;  0A8h - mov   - One Byte Long
  db      2                     ;  0A9h - mov   - One Byte Long
  db      2                     ;  0AAh - mov   - One Byte Long
  db      2                     ;  0ABh - mov   - One Byte Long
  db      2                     ;  0ACh - mov   - One Byte Long
  db      2                     ;  0ADh - mov   - One Byte Long
  db      2                     ;  0AEh - mov   - One Byte Long
  db      2                     ;  0AFh - mov   - One Byte Long
  db      2                     ;  0B0h - anl   - One Byte Long
  db      2                     ;  0B1h - acall - One Byte Long
  db      2                     ;  0B2h - cpl   - One Byte Long
  db      1                     ;  0B3h - cpl   - One Byte Long
  db      3                     ;  0B4h - cjne  - One Byte Long
  db      3                     ;  0B5h - cjne  - One Byte Long
  db      3                     ;  0B6h - cjne  - One Byte Long
  db      3                     ;  0B7h - cjne  - One Byte Long
  db      3                     ;  0B8h - cjne  - One Byte Long
  db      3                     ;  0B9h - cjne  - One Byte Long
  db      3                     ;  0BAh - cjne  - One Byte Long
  db      3                     ;  0BBh - cjne  - One Byte Long
  db      3                     ;  0BCh - cjne  - One Byte Long
  db      3                     ;  0BDh - cjne  - One Byte Long
  db      3                     ;  0BEh - cjne  - One Byte Long
  db      3                     ;  0BFh - cjne  - One Byte Long
  db      2                     ;  0C0h - push  - One Byte Long
  db      2                     ;  0C1h - ajmp  - One Byte Long
  db      2                     ;  0C2h - clr   - One Byte Long
  db      1                     ;  0C3h - clr   - One Byte Long
  db      1                     ;  0C4h - swap  - One Byte Long
  db      2                     ;  0C5h - xch   - One Byte Long
  db      1                     ;  0C6h - xch   - One Byte Long
  db      1                     ;  0C7h - xch   - One Byte Long
  db      1                     ;  0C8h - xch   - One Byte Long
  db      1                     ;  0C9h - xch   - One Byte Long
  db      1                     ;  0CAh - xch   - One Byte Long
  db      1                     ;  0CBh - xch   - One Byte Long
  db      1                     ;  0CCh - xch   - One Byte Long
  db      1                     ;  0CDh - xch   - One Byte Long
  db      1                     ;  0CEh - xch   - One Byte Long
  db      1                     ;  0CFh - xch   - One Byte Long
  db      2                     ;  0D0h - pop   - One Byte Long
  db      2                     ;  0D1h - acall - One Byte Long
  db      2                     ;  0D2h - setb  - One Byte Long
  db      1                     ;  0D3h - setb  - One Byte Long
  db      1                     ;  0D4h - da    - One Byte Long
  db      3                     ;  0D5h - djnz  - One Byte Long
  db      1                     ;  0D6h - xchd  - One Byte Long
  db      1                     ;  0D7h - xchd  - One Byte Long
  db      2                     ;  0D8h - djnz  - One Byte Long
  db      2                     ;  0D9h - djnz  - One Byte Long
  db      2                     ;  0DAh - djnz  - One Byte Long
  db      2                     ;  0DBh - djnz  - One Byte Long
  db      2                     ;  0DCh - djnz  - One Byte Long
  db      2                     ;  0DDh - djnz  - One Byte Long
  db      2                     ;  0DEh - djnz  - One Byte Long
  db      2                     ;  0DFh - djnz  - One Byte Long
  db      1                     ;  0E0h - movx  - One Byte Long
  db      2                     ;  0E1h - ajmp  - One Byte Long
  db      1                     ;  0E2h - movx  - One Byte Long
  db      1                     ;  0E3h - movx  - One Byte Long
  db      1                     ;  0E4h - clr   - One Byte Long
  db      2                     ;  0E5h - mov   - One Byte Long
  db      1                     ;  0E6h - mov   - One Byte Long
  db      1                     ;  0E7h - mov   - One Byte Long
  db      1                     ;  0E8h - mov   - One Byte Long
  db      1                     ;  0E9h - mov   - One Byte Long
  db      1                     ;  0EAh - mov   - One Byte Long
  db      1                     ;  0EBh - mov   - One Byte Long
  db      1                     ;  0ECh - mov   - One Byte Long
  db      1                     ;  0EDh - mov   - One Byte Long
  db      1                     ;  0EEh - mov   - One Byte Long
  db      1                     ;  0EFh - mov   - One Byte Long
  db      1                     ;  0F0h - movx  - One Byte Long
  db      2                     ;  0F1h - acall - One Byte Long
  db      1                     ;  0F2h - movx  - One Byte Long
  db      1                     ;  0F3h - movx  - One Byte Long
  db      1                     ;  0F4h - cpl   - One Byte Long
  db      2                     ;  0F5h - mov   - One Byte Long
  db      1                     ;  0F6h - mov   - One Byte Long
  db      1                     ;  0F7h - mov   - One Byte Long
  db      1                     ;  0F8h - mov   - One Byte Long
  db      1                     ;  0F9h - mov   - One Byte Long
  db      1                     ;  0FAh - mov   - One Byte Long
  db      1                     ;  0FBh - mov   - One Byte Long
  db      1                     ;  0FCh - mov   - One Byte Long
  db      1                     ;  0FDh - mov   - One Byte Long
  db      1                     ;  0FEh - mov   - One Byte Long
