;  FullRTOS - Digital Clock/Thermometer using FullRTOS  
;
;
;  This Application implements a Real Time Clock and Thermometer, 
;   outputting the data onto an LCD Display and updating the clock
;   using a button input.
;
;  The Tasks Implemented are:
;   Application/TmrInitTask - Initialize/Start Up the Timer Interrupt and Load
;                              Other Tasks
;   LCDTask                 - Execute the Command Passed to it
;   ClockTask               - Real Time Clock Task
;   TemperatureTask         - Task Used for Checking Temperature
;
;
;  Myke Predko
;  98.04.28
;
;  Hardware Notes:
;  DS80C320 Running at 4 MHz
;  LCD Data Bits are on P3
;  LCD.6 - "E" Clock on P1.7
;  LCD.5 - "RW" Pin on P1.5
;  LCD.4 - "RS" Pin on P1.4
;  Time Set Button - on P1.0
;  Temperature Pin - P1.0


;  Constant Declarations
;   - These are to be changed for Different MCUs
_TaskStart EQU 030h		;  Tasks Start at 030h
_TaskEnd   EQU 070h
_TIBSize   EQU 010h             ;  Size of Each Task Information Block

;  Variable Declarations
_Semaphore EQU 020h 		;  The Semaphore Byte
_CurTask   EQU 004h		;  Index of the Currently Executing Task

;  RTOS Function Macros
MACRO NextTask			;  Let the Next Task Execute

  acall	 _RTOSNextTask

ENDMAC

MACRO EndTask			;  End the Current Task

  mov	 R0,_CurTask		;  Point to the Current Task
  mov	 @R0,#0FFh		;  Mark in the Task Block that the Task has Ended

  acall	 _RTOSNextTask

ENDMAC

MACRO StartTask (Task,Priority)	;  Start the New Task

  mov	 R2,#(Task AND 255)
  mov	 R1,#(Task SHR 8)
  mov    R0,Priority

  acall	 _RTOSStartTask		;  Task Number in R0 (or -1)

ENDMAC

MACRO WaitInt(Number)		;  Wait for the Specific Interrupt Number

  mov	 R0,_CurTask		;  Get the Pointer to the Task
  mov	 @R0,#(040h + Number)

  acall	 _RTOSNextTask		;  Now, Start Waiting for the Other Tasks

ENDMAC

MACRO RelSem(Number)		;  Release the Semaphore Number

  setb	 Number			;  Just Set the Semaphone Number

ENDMAC

MACRO GetSem(Number)		;  Get the Semaphore

@_RTOSGetSem:			;  Loop Back Here for Each Semaphore

  jnb	 Number,@_RTOSGetSemGot	;  Semaphore Gotten

  acall	 _RTOSNextTask		;  Wait for the Next Task

  ajmp	 @_RTOSGetSem		;  Try to get the Semaphore Again

@_RTOSGetSemGot:		;  Have the Semaphore

ENDMAC

MACRO AckMsg(Number)		;  Acknowledge the Message from Task #

  mov	 A,Number

  acall	 _RTOSAckMsg

@_RTOSAckMsg:

ENDMAC

MACRO ReadMsg(Number)		;  Read the Message from the Task

  mov	 A,Number
  acall	 _RTOSReadMsg

ENDMAC

MACRO SendMsg(Task,Ptr)	        ;  Send the Message Pointed to 

  mov	 R0,_CurTask		;  Point to the Current Task
  mov    A,Task
  orl    A,#%00010000
  mov    @R0,A
  anl    A,#(%00010000 XOR 0FFh);  Load the Accumulator with the Task Number

  mov	 R0,Ptr			;  Pointer can be Value or Address

  acall	 _RTOSSendMsg

ENDMAC

MACRO WaitMsg			;  Wait for the Message to be Send

  acall	 _RTOSWaitMsg

ENDMAC


;  Mainline
 org 0                          ;  Execution Starts Here

  ajmp	 _RTOSMain		;  Jump to the RTOS Initialization Code


;  Interrupt Handlers
 org 0003h			;  Int 0 - _Int0 Interrupt

  push	 IE			;  Save the Interrupt Register 

  push	 ACC			;  Save the Accumulator

  mov	 A,#%01000000

  ajmp	 _RTOSInt		;  Handle the Interrupt

 org 000Bh			;  Int 1 - Timer0 Interrupt

  push	 IE

  push	 ACC			;  Save the Accumulator

  mov	 A,#%01000001

  ajmp	 _RTOSInt		;  Handle the Interrupt

 org 0013h			;  Int 2 - _Int1 Interrupt

  push	 IE

  push	 ACC			;  Save the Accumulator

  mov	 A,#%01000010

  ajmp	 _RTOSInt		;  Handle the Interrupt

 org 001Bh			;  Int 3 - Timer1 Interrupt

  push	 IE

  push	 ACC			;  Save the Accumulator

  mov	 A,#%01000011		;  Set the Interrupt Number

_RTOSInt:			;  Handle the Interrupt Reqests

  clr	 IE.7			;  Disable Global Interrupts

  push	 PSW			;  Save the Task's Context Information
  push	 0			;  All Tasks run out of Bank 0 Primarily
  push	 1
  push	 2

  mov    R1,A                   ;  Save the Interrupt Number

  mov	 R0,_CurTask		;  Get the Pointer to the Current Task
  inc	 R0			;   Point to the SP Byte
  inc    R0
  mov	 @R0,SP			;  Save the Stack Pointer

  mov	 R0,#_TaskStart		;  Now, Look for Tasks Waiting for the Interrupt

_RTOSIntLoop:

  mov    A,R1                   ;  Get the Interrupt Number

  xrl	 A,@R0			;  Do we Have a Match at This Task?  
  jnz	 _RTOSIntLoopSkip	
  mov	 @R0,A			;  Yes, Mark it as able to Execute
				;   (Load with the Zero in "A")
_RTOSIntLoopSkip:		

  mov	 A,#_TIBSize		;  Point to the Next Task
  add	 A,R0
  mov	 R0,A

  cjne	 A,#_TaskEnd,_RTOSIntLoop

  ajmp	 _RTOSNextTaskDo	;  It's not, Jump to the Next Task Code

_RTOSStartTask:			;  Start the Specified Task

  push	 IE			;  Save the Context Registers
  clr	 IE.7
  push	 ACC			     
  push	 PSW
  push	 0
  push	 1
  push	 2

  mov	 R1,_CurTask		;  Keep track of where we are.
  inc	 R1
  inc    R1	
  mov	 @R1,SP			;  Save the Stack Pointer

  mov	 R1,SP			;  Save the Stack Pointer for Later

  mov	 R0,#(_TaskStart - _TIBSize)  ;  Get Address of the First Task

_RTOSStartTaskLoop:		;  Now, Loop Around Looking for an Available Task

  mov	 A,R0
  add	 A,#_TIBSize		;  Look at the Next Task
  mov	 R0,A

  add	 A,#((_TaskEnd XOR 0FFh) + 1)
  jz	 RTOSStartTaskNo	;  Don't have a Task to Put it in, End

  cjne	 @R0,#0FFh,_RTOSStartTaskLoop  ;  Jump if Task Not Available

  mov	 @R0,#0			;  Enable the Task to Run
  mov	 _CurTask,R0		;  Save the Current Task Address
  mov	 A,R0			;  Now, Set up the Task
  inc    A
  inc    A
  mov	 SP,A			;  Have a New Stack Pointer

  mov	 A,@R1			;  Put the Return Address on the Stack
  push	 ACC
  dec	 R1
  mov	 A,@R1
  push	 ACC
  dec	 R1

  inc    R0
  mov    A,@R1                  ;  Save the Task's Priority
  mov    @R0,A

  mov	 A,_CurTask		;  Save the Task Number
  add	 A,#((_TaskStart XOR 0FFh) + 1)
  swap	 A			;  Put the Task Number in the Correct Format
  mov	 @R1,A			;   Into R0 of the Previous Task

  mov	 IE,#08Fh		;  Enable Interrupts

  ret

RTOSStartTaskNo:                ;  No Task is Available

  dec    R1                     ;  Point to R0
  dec    R1
  mov	 @R1,#0FFh		;  Mark that Nothing was Available

  mov	 R0,_CurTask		;  Get the Pointer to the Current Task Set

  ajmp   _RTOSExecuteTask       ;  Now, Start up the Task

_RTOSNextTask:			;  Jump to the Next Active Task

  push	 IE			;  Save the Context Registers
  clr	 IE.7			;  Disable Global Interrupts  
  push	 ACC			     
  push	 PSW
  push	 0
  push	 1
  push	 2

  mov    R0,_CurTask            ;  Save the Stack Pointer
  inc    R0
  inc    R0
  mov    @R0,SP

_RTOSNextTaskDo:		;  Assume that Ints are off and 
				;   Context Registers are Saved
  mov	 R0,_CurTask		;  Start at the Current Task
  mov    R1,#_TaskEnd           ;  Find the Next Highest Task to Execute
  mov    R2,#0FFh

  ajmp   _RTOSNextTaskInc       ;  Jump to the Next Task

_RTOSNextTaskLoop:

  mov	 A,R0
  xrl    A,_CurTask             ;  Have We Looped Through them All?
  jz	 _RTOSExecuteTask

_RTOSNextTaskInc:               ;  Increment to the Next Task

  cjne   R0,#_TaskEnd,_RTOSNextTaskSkip

  mov	 R0,#(_TaskStart - _TIBSize)  ;  Yes, Start at Beginning of the Buffer

_RTOSNextTaskSkip:		;  Point to the Next Task

  mov	 A,#_TIBSize		
  add	 A,R0
  mov	 R0,A

  cjne	 @R0,#0,_RTOSNextTaskLoop  ;  Jump Around and Do the Loop Again

  inc    R0                     ;  Point to the Next Task
  mov    A,@R0
  dec    R0
  clr    C
  subb   A,R2

  jnc    _RTOSNextTaskLoop      ;  Same or Lower Priority

  add    A,R2                   ;  New Task
  mov    R2,A                   ;  Save the New Priority

  mov    A,R0                   ;  Save the New Task Information Block
  mov    R1,A

  ajmp   _RTOSNextTaskLoop

_RTOSExecuteTask:		;  Start the Task Executing

  mov	 _CurTask,R1		;  Save the Current Task Pointer

  inc	 R1			;  Get the Tasks Stack Pointer
  inc    R1
  mov	 SP,@R1

  pop	 2			;  Restore the Context Registers
  pop	 1
  pop	 0
  pop	 PSW
  pop	 ACC
  pop	 IE

  reti

_RTOSAckMsg:			;  Acknowledge the Task Message

  swap	 A			;  Get the Offset to the Task to Ack
  add	 A,#_TaskStart		
  mov	 R0,A

  mov	 A,_CurTask		;  Get the Current Task
  add	 A,#((_TaskStart XOR 0FFh) + 1)  ;  Figure Out it's Number
  swap	 A
  orl	 A,#_TIBSize		;  Make Sure in the Correct Format

  xrl	 A,@R0			;  Is the Task Waiting for the Current one?
  jnz	 _RTOSAckMsgEnd		;  Nope, Just End

  mov	 @R0,#0			;  Yes it is, Allow it to Execute

_RTOSAckMsgEnd:

  ret

_RTOSReadMsg:			;  Read the Message from the Task in A

  swap	 A			;  Get the Offset to the Task to Ack
  add	 A,#_TaskStart		
  mov	 R0,A

  mov	 A,_CurTask		;  Get the Current Task
  add	 A,#((_TaskStart XOR 0FFh) + 1)  ;  Figure Out it's Number
  swap	 A
  orl	 A,#010h		;  Make Sure in the Correct Format

  xrl	 A,@R0			;  Is the Task Waiting for the Current one?
  mov	 R1,A

  mov    A,#0FFh		;  Assume it will fail 

  cjne	 R1,#0,_RTOSReadMsgEnd	;  Task doesn't have a Message, Return - in ACC

  inc	 R0			;  Now, Figure out Where the Message is
  inc    R0
  mov	 A,@R0                  ;  Get the Task's Stack Pointer
  add    A,#0FEh                ;  Get the Value in the Sending Task's R1
  mov	 R0,A                   ;  Get the Value in the Task's R1 Stack Value 

  mov	 A,@R0			;  Return the Value in R0 in the Accumulator

_RTOSReadMsgEnd:

  ret

_RTOSSendMsg:			;  Send the Message to the Task

  push	 IE			;  Save all the Registers
  clr	 IE.7
  push	 ACC
  push	 PSW
  push	 0
  push	 1
  push	 2

  mov    R0,_CurTask            ;  Save the Current Stack Pointer
  inc    R0
  inc    R0
  mov    @R0,SP

  swap	 A			;  Get the Task Address
  add	 A,#_TaskStart
  mov	 R1,A

  cjne	 @R1,#%00100000,_RTOSNextTaskDo  ;  Jump to the Next Task

  mov	 @R1,#0			;  Enable Task Sending to

  ajmp	 _RTOSNextTaskDo	;  Jump to the Next Task

_RTOSWaitMsg:			;  Waiting for a Message, See if Something There First

  push	 IE			;  Save all the Registers
  clr	 IE.7
  push	 ACC
  push	 PSW
  push	 0
  push	 1
  push	 2

  mov    R0,_CurTask            ;  Save the Stack Pointer
  inc    R0
  inc    R0
  mov    @R0,SP

  mov	 R0,#(_TaskStart - _TIBSize)

  mov	 A,_CurTask
  add	 A,#((_TaskStart XOR 0FFh) + 1)
  swap	 A
  orl	 A,#010h		;  "A" has Message to Find

_RTOSWaitMsgLoop:		;  See if Any Tasks Are Sending to Current

  xrl	 A,@R0			;  Do we Have a Task Sending to This one?
  jz	 _RTOSWaitMsgHere	;  If Zero, then "Yes"

  xrl	 A,@R0			;  Restore the Accumulator

  xch	 A,R0
  add	 A,#010h		;  Add 10 to the Task Pointer
  xch	 A,R0

  cjne	 R0,#_TaskEnd,_RTOSWaitMsgLoop  ;  Try the Next Task

  mov	 R0,_CurTask		;  Else, Mark the Task as Blocking

  mov	 @R0,#%00100000

  ajmp	 _RTOSNextTaskDo

_RTOSWaitMsgHere:               ;  Something is Waiting on the Message

  mov    R0,_CurTask            ;  Reset the Task as Executable

  mov    @R0,#0

  ajmp   _RTOSNextTaskDo
  

_RTOSMain:			;  RTOS Initialization Code

  mov	 _Semaphore,#00Fh	;  Clear the Semaphores (Set the bits)

  mov	 R0,#_TaskStart		;  Mark the Tasks as NOT Executing

_RTOSMainLoop:

  mov	 @R0,#0FFh		;  Mark the Task as NOT Available

  mov	 A,#_TIBSize		;  Point to the Next Task
  add	 A,R0
  mov	 R0,A

  cjne	 A,#_TaskEnd,_RTOSMainLoop  ;  At the Last Task Block?

  mov	 @R0,#0			;  Yes, Mark that the Task can Run

  mov    _CurTask,R0            ;  Note that it is the Current Task

  inc    R0                     ;  Point to the Data in Memory
  mov    @R0,#0FFh              ;  AlwaysTask Lowest Priority
  inc    R0
  inc    R0
  mov	 @R0,#(_AlwaysTask AND 255)
  inc    R0
  mov	 @R0,#(_AlwaysTask SHR 8)

  mov	 SP,R0                  ;  Save the Task's Stack Pointer

  mov	 IE,#%10001111		;  Enable all Interrupts

  ret				;  Jump to the _AlwaysTask 


;  Always Executable Task

_AlwaysTask:			;  The Always Available Task

  StartTask (Application,#080h)	;  Start the Application Task

_AlwaysTaskLoop:		;  Loop Here Forever

  NextTask			;  Look at the Next Task

  ajmp	 _AlwaysTaskLoop	;  Loop Around (with Interrupts Enabled)



;  ##### - Put in "$include app.asm" Load Here

;  Define Sample Application Variables
LCDTask  EQU 010h               ;  LCDTask Number
TimeTask EQU 011h               ;  Time Task Number
TempTask EQU 012h               ;  Temperature Task Number

TimeLo   EQU 013h               ;  24 Bit Counter Value
TimeHi   EQU 014h               
TimeFrac EQU 015h

Hours    EQU 016h               ;  Actual Time
Minutes  EQU 017h
Seconds  EQU 018h

Increment EQU 019h              ;  Increment Value

TempWait EQU 01Ah               ;  Wait 2 Seconds for Each Temperature Update
TempCount EQU 01Bh              ;  Bit Counter for the Temperature
TempSP0  EQU 01Ch               ;  Data Written to/Read from the DS1820
TempSP1  EQU 01Dh
TempDlay EQU 01Eh               ;  Delay Value for DS1820 Operations

Application:                    ;  Simple Test Application

  mov    CKCON,#%00001000       ;  Use Internal /4 Clock for Timer0
  mov    TMOD,#%00000001        ;  Timer0 - Uses Internal Clock
                                ;         - Run in Mode 1
  mov    TCON,#%00010101        ;  Start Timer0 running

  StartTask (LCDTaskStart,#0C0h);  Start the LCD Task
  mov    LCDTask,R0

  StartTask (TimeTaskStart,#000h);  Start the Time Task
  mov    TimeTask,R0

  StartTask (TempTaskStart,#040h);  Start the Temperature Task
  mov    TempTask,R0

  EndTask                       ;  End the Application Task and Let Other Tasks Execute


TimeTaskStart:                  ;  The beginning of the Timer Task

  mov    Hours,#0               ;  Initialize the RTC Values
  mov    Minutes,#0
  mov    Seconds,#0

  SendMsg (LCDTask,#10)         ;  Send an LCD Time Update Request

  mov    TimeLo,#0              ;  The 24 Bit Second Fraction
  mov    TimeHi,#0
  mov    TimeFrac,#0

  mov    Increment,#1           ;  Only Increment By 1 (Button To Change)

TimeTaskLoop:                   ;  Loop Around Here once each Second 

  WaitInt(1)                    ;  Wait 65,536 usecs

  inc    TimeFrac               ;  Increment the Current Fraction of a Second

  mov    A,TimeFrac             ;  Have we Counted 16x?
  cjne   A,#16,TimeTaskLoop     ;  If we Don't have 16, then "no"

  mov    A,TimeLo               ;  Take 1,000,000 From the Current Seconds 
  clr    C
  subb   A,#040h
  mov    TimeLo,A

  mov    A,TimeHi               ;  Now, Subtract the Middle 8 Bits
  subb   A,#042h
  mov    TimeHi,A

  mov    A,TimeFrac             ;  Reset the Time Fraction
  subb   A,#00Fh                ;  Take Away 15 From the Fraction
  mov    TimeFrac,A

  inc    Seconds                ;  Increment the Seconds

  jb     P1.0,TimeSecCheck      ;  If P1.0 is Time Set Button 

  mov    Seconds,#60            ;  If the Button is Pressed, Increment the Minutes 

TimeSecCheck:                   ;  Check to See if the Seconds are > 59 

  mov    A,Seconds              
  clr    C
  subb   A,#60                  ;  Do we Have 60 Seconds?
  jnc    TimeMinIncrement       ;  If Carry Set, then No

  mov    Increment,#1           ;  Reset The Increment Value to the Start Value
  
  ajmp   TimeTaskLoop           ;  Wait another Second  

TimeMinIncrement:

  mov    Seconds,#0             ;  Reset the Seconds

  mov    A,Increment            ;  Update the Minutes
  add    A,Minutes
  mov    Minutes,A

  clr    C                      ;  Are Minutes > 59?
  subb   A,#60
  jc     TimeDisplay            ;  If Carry Set, No - Display the Time

  mov    Minutes,#0

  inc    Hours                 ;  Increment the Hours

  mov    A,Hours               ;  Have we Looped Around?
  clr    C
  subb   A,#24
  jc     TimeDisplay

  mov    Hours,#0              ;  Reset the Hours  

TimeDisplay:                   ;  Update the LCD's Time Display

  SendMsg (LCDTask,#10)        ;  Send an LCD Time Update Request

  jb     P1.0,TimeTaskLoop     ;  If the Button is High, Then Return

  mov    A,Increment           ;  Set a New Increment Value
  setb   C                     ;  Want to Move Up to 63 on the Increments
  rlc a
  anl    A,#03Fh               ;  Leave with no more than 63 for the Increment
  mov    Increment,A

  ajmp   TimeTaskLoop



TempTaskStart:                  ;  The beginning of the DS1820 Temperature Sensor Task

  mov    TempWait,#0            ;  Reset TempWait

TempTaskLoop:                   ;  Loop Around Here for Each Task

  WaitInt (1)                   ;  Going To Wait 2 Seconds before Executing

  inc    TempWait               ;  Increment the Wait Variable

  mov    A,TempWait             ;  Have we Gone 2 Seconds Yet?
  cjne   A,#32,TempTaskLoop     ;  If Not Equal to 32, Then Loop Around Again

  mov    TempWait,#0

  acall  DSReset                ;  Reset the DS1820
  mov    A,#0CCh                ;  Start the Temperature Read Operation
  acall  SendDS
  mov    A,#044h
  acall  SendDS

  mov    TempDlay,#078h         ;  Wait for the Operation to Execute
TempWait1:
  djnz   TempDlay,TempWait1

  acall  DSReset                ;  Now, Read the Temperature
  mov    A,#0CCh
  acall  SendDS
  mov    A,#0BEh
  acall  SendDS

  mov    TempDlay,#01Eh
TempWait2:
  djnz   TempDlay,TempWait2

  acall  GetDS
  mov    TempSP0,A

  mov    TempDlay,#01Eh
TempWait3:
  djnz   TempDlay,TempWait3

  acall  GetDS
  mov    TempSP1,A

  mov    TempDlay,#01Eh
TempWait4:
  djnz   TempDlay,TempWait4

  acall  DSReset                ;  Reset the DS1820

  SendMsg (LCDTask,#4)

  ajmp   TempTaskLoop

DSReset:                        ;  Reset the DS1820

  clr    P0.1                   ;  Drop the DS1820's Line 

  mov    TempDlay,#078h         ;  Wait 480 usecs
DSResetWait1:
  djnz   TempDlay,DSResetWait1

  setb   P0.1

DSResetWait2:                   ;  Now, Wait 1 msec
  djnz   TempDlay,DSResetWait2

  ret

SendDS:                         ;  Send the Byte in "A" to the DS1820

  mov    TempCount,#8           ;  Sending 8 Bits

  clr    IE.7                   ;  Disable Interrupts During this Operation

SendDSLoop:                     ;  Loop Here for Each Bit

  rrc a                         ;  Put the LSB into the Carry Flag

  clr    P0.1                   ;  Drop the Line Low

  jnc    SendDSDlay             ;  Do we Delay the Line low?

  setb   P0.1                   ;  No, Sending a "1"

SendDSDlay:                     ;  Delay 30 Instruction Cycles before Setting Pin
  mov    R0,#9
SendDSWait:
  djnz   R0,SendDSWait

  setb   P0.1

  mov    R0,#7                  ;  Wait 21 Cycles before sending the next bit
SendDSWait2:
  djnz   R0,SendDSWait2

  djnz   TempCount,SendDSLoop

  setb   IE.7                   ;  Enable Interrupts Again

  ret

GetDS:                          ;  Get 8 Bits from the DS1820

  mov    TempCount,#8           ;  Sending 8 Bits

  clr    IE.7                   ;  Disable Interrupts During this Operation

GetDSLoop:                      ;  Loop Here for Each Bit

  clr    P0.1                   ;  Toggle the I/O Pin
  setb   P0.1

  nop

  setb   C                      ;  Now, See what is Returned
  jb     P0.1,GetDSSave         ;  Is the Bit Still High?
  clr    C
GetDSSave:                      ;  Save the Carry Flag as the Bit Read
  rrc a

  mov    R0,#5                  ;  Delay 16 Cycles for the DS1820 to Reset
GetDSWait:
  djnz   R0,GetDSWait

  djnz   TempCount,GetDSLoop

  setb   IE.7                   ;  Enable Interrupts Again

  ret


LCDTaskStart:                   ;  LCD Task, Display Data on the LCD

  clr    P1.7                   ;  Turn Off the LCD Pins
  clr    P1.5
  clr    P1.4

  WaitInt (1)                   ;  Wait 20 mscec+ for the LCD to Reset
  WaitInt (1)

  mov    P3,#030h               ;  Reset the LCD
  setb   P1.7                   ;  Toggle "E" Clock
  clr    P1.7

  WaitInt (1)                   ;  Wait for the Command to Take

  setb   P1.7                   ;  Toggle "E" Clock Again
  clr    P1.7

  WaitInt (1)                   ;  Delay for the Command to Complete

  setb   P1.7                   ;  Toggle "E" one Last Time
  clr    P1.7

  WaitInt (1)

  mov    A,#038h                ;  Want an 8 Bit LCD Operation
  acall  SendINS

  mov    A,#010h                ;  Turn Off the LCD
  acall  SendINS

  mov    A,#001h                ;  Clear the LCD
  acall  SendINS

  WaitInt(1)                    ;  Wait 500 mSecs After Previous

  mov    A,#00Ch                ;  Turn on the Display
  acall  SendINS

  mov    A,#'T'                 ;  Put the "Time: xx:xx XX" String on the Display
  acall  SendCHAR
  mov    A,#'i'
  acall  SendCHAR
  mov    A,#'m'
  acall  SendCHAR
  mov    A,#'e'
  acall  SendCHAR
  mov    A,#':'
  acall  SendCHAR
  mov    A,#' '
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#':'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#' '
  acall  SendCHAR
  mov    A,#'X'
  acall  SendCHAR
  mov    A,#'X'
  acall  SendCHAR

  mov    A,#0C0h                ;  Put in "Temp: xxxx C"
  acall  SendINS
  mov    A,#'T'
  acall  SendCHAR
  mov    A,#'e'
  acall  SendCHAR
  mov    A,#'m'
  acall  SendCHAR
  mov    A,#'p'
  acall  SendCHAR
  mov    A,#':'
  acall  SendCHAR
  mov    A,#' '
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#'x'
  acall  SendCHAR
  mov    A,#' '
  acall  SendCHAR
  mov    A,#'C'
  acall  SendCHAR

LCDTaskLoop:                    ;  Loop Here and Process Each Command Sent to LCD

  WaitMsg                       ;  Wait for a message to be sent to it

  ReadMsg (TimeTask)
  xrl    A,#0FFh                ;  Do We Have a Message to Update the Time?
  jz     LCDTempRead            ;  No, Update the Temperature

  AckMsg (TimeTask)             ;  Acknowledge the Message

  mov    A,#089h                ;  Display the Time
  acall  SendINS

  mov    A,Minutes              ;  Display the Minutes First
  mov    B,#10
  div ab
  add    A,#'0'
  acall  SendCHAR
  mov    A,B
  add    A,#'0'
  acall  SendCHAR

  mov    A,#086h                ;  Now, Display the Hours
  acall  SendINS

  mov    A,Hours                ;  Do We Have AM or PM?
  clr    C
  subb   A,#12
  jc     LCDHaveHours           ;  AM, Live With It

  subb   A,#12                  ;  Convert to PM

LCDHaveHours:                   ;  Now, Do we Have Noon/Midnight?

  add    A,#12                  ;  Restore To Normal Time

  jnz    LCDDisplayHours        ;  No, Display the Hours

  mov    A,#12

LCDDisplayHours:                ;  Display the Hours

  mov    B,#10                  ;  Convert to Decimal
  div ab
  add    A,#'0'
  acall  SendCHAR
  mov    A,B
  add    A,#'0'
  acall  SendCHAR

  mov    A,#08Ch                ;  Now, Display "AM" or "PM"
  acall  SendINS

  mov    A,Hours
  clr    C
  subb   A,#12
  mov    A,#'A'                 ;  Start with "AM"
  jc     LCDDisplayAPM
  mov    A,#'P'
LCDDisplayAPM:
  acall  SendCHAR

  mov    A,#'M'
  acall  SendCHAR
   
  ajmp   LCDTaskLoop            ;  Wait for the Next Request

LCDTempRead:                    ;  Read the Temperature Passed to the Task

  ReadMsg (TempTask)

  mov    A,#0C6h                ;  Move the Cursor to Temperature Part of the Display
  acall  SendINS

  mov    A,TempSP1              ;  Get the Positive/Negative Temperature
  mov    C,ACC.0                ;  Marked in Bit 0 of SP1
  mov    A,#' '                 ;  Assume Positive and Can Display
  jnc    LCDTempDisplay

  clr    A                      ;  Invert the Temperature
  clr    C
  subb   A,TempSP0
  mov    TempSP0,A

  mov    A,#'-'                 ;  Mark as Negative

LCDTempDisplay:                 ;  Display the Temperature

  acall  SendCHAR

  mov    A,TempSP0              ;  Now, Display the Temperature

  clr    C                      ;  Divide Temperature by 2 to get Celcius
  rrc    A

  mov    B,#100                 ;  Get the Hundreds of Degrees
  div ab
  add    A,#'0'
  acall  SendCHAR

  mov    A,B                    ;  Get the tens of Degrees
  mov    B,#10
  div ab
  add    A,#'0'
  acall  SendCHAR

  mov    A,B                    ;  Display the Ones of Degrees
  add    A,#'0'
  acall  SendCHAR

  AckMsg (TempTask)             ;  Acknowledge the Message

  ajmp   LCDTaskLoop


SendINS:                        ;  Send Instruction to the LCD

  setb   P1.5                   ;  Set the RW Line
  clr    P1.4                   ;  Clear the RS Line

  setb   P3.7                   ;  Make Sure the Polling Line is High

SendINSLoop:                    ;  Loop Here Until the LCD is Active

  setb   P1.7                   ;  Enable the "E" Flag

  jnb    P3.7,SendINSSkip       ;  If Bit 7 is Low, Send the Character

  clr    P1.7                   ;  Turn off the "E" Flag

  NextTask                      ;  Else, Loop Through the Operation

  ajmp   SendINSLoop

SendINSSkip:                    ;  Now, We can Output the Character

  clr    P1.7                   ;  Turn off the "E" Bit
  clr    P1.5                   ;  Turn off the LCD Read Operation

  mov    P3,A                   ;  Output the Character to Send
  setb   P1.7                   ;  Toggle the "E" Flag
  clr    P1.7

  ret                           ;  Return to the Caller


SendCHAR:                        ;  Send Character to the LCD

  setb   P1.5                   ;  Set the RW Line
  clr    P1.4                   ;  Clear the RS Line

  setb   P3.7                   ;  Make Sure the Polling Line is High

SendCHARLoop:                   ;  Loop Here Until the LCD is Active

  setb   P1.7                   ;  Enable the "E" Flag

  jnb    P3.7,SendCHARSkip      ;  If Bit 7 is Low, Send the Character

  clr    P1.7

  NextTask                      ;  Else, Loop Through the Operation

  ajmp   SendCHARLoop

SendCHARSkip:                   ;  Now, We can Output the Character

  clr    P1.7                   ;  Turn off the "E" Flag
  clr    P1.5                   ;  Turn off the LCD Read Operation
  setb   P1.4                   ;  Send out the Character  

  mov    P3,A                   ;  Output the Character to Send
  setb   P1.7                   ;  Toggle the "E" Flag
  clr    P1.7

  ret                           ;  Return to the Caller
